/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.client.internal.Attribute");

dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.ConfigurationParameters");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");

(function() {
	
var ConfigurationParameters= com.ibm.team.workitem.web.client.internal.ConfigurationParameters;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;
	
dojo.declare("com.ibm.team.workitem.web.client.internal.Attribute", null, {
	
	// API -------------------------------------------
	
	constructor: function(attribute) {
		
		this._attribute= attribute;
		this._valueProviderCfg= null;
		this._ValueProviderCtor= null;
		
		if (attribute.valueProvider) {
			var props= com.ibm.team.workitem.web.client.util.buildMap(attribute.valueProvider.properties);
			var cfg= new ConfigurationParameters(dojo.fromJson(props.configuration));
			this._valueProviderCfg= cfg;
		}
	},
	
	getIdentifier: function() {
		return this._attribute.id;
	},
	
	getLabel: function(name) {
		return this._attribute.label;
	},
	
	getAttributeTypeId: function() {
		return this._attribute.attributeTypeId;
	},
	
	dependsOn: function(attributeId) {
		var dependencies= this._attribute.dependentAttributes || [];
		return dojo.indexOf(dependencies, attributeId) > -1;
	},
	
	hasValueProvider: function() {
		return this._valueProviderCfg != null;
	},
	
	getValue: function(/* WorkItem */ workItem) {
		var attributeId= this.getIdentifier();
		var valueProvider= this._getValueProvider();
		if (valueProvider) {
			try {
				return valueProvider.getValue(attributeId, workItem, this._valueProviderCfg);
			} catch (e) {
				console.error(e);
			}
		}	
		return workItem.getValue(attributeId);
	},
	
	getValueSet: function(/* WorkItem */ workItem, responseHandler) {
		
		var args= { 
			attributeName: this.getIdentifier(),
			projectAreaItemId: workItem.getProjectAreaItemId(),
			typeId: workItem.getWorkItemTypeId()
		};
		
		var dependencies= this._attribute.dependentAttributes;
		if (dependencies) {
			
			// use the same format as a work item post
			var identifiers= new Array();
			var values= new Array();
			
			for (var i= 0; i < dependencies.length; i++) {
				var dependencyValue= workItem.getValue(dependencies[i]);
				if (dependencyValue) {
					identifiers.push(dependencies[i]);
					values.push(dependencyValue);
				}
			}
			args.attributeIdentifiers= identifiers;
			args.attributeValues= values;
		}
		WorkItemClient.getValues(responseHandler, args);
	},
	
	// Internal --------------------------------------
	
	_getPath: function() {
		return ["attributes", this.getIdentifier()];
	},
	
	_getValueProvider: function() {
		if (this._ValueProviderCtor == null && this._valueProviderCfg != null) {
			var provider= this._valueProviderCfg.getChild("script");
			var clazz= provider ? provider.getString("class") : null;
			var ctor= null;
			if (clazz != null) {
				ctor= dojo.getObject(clazz);
				if (ctor == null) {
					jazz.core.loader.load_sync(clazz);
					ctor= dojo.getObject(clazz);
				}
			}
			this._ValueProviderCtor= ctor;
		}
		if (this._ValueProviderCtor) {
			return new this._ValueProviderCtor();
		}
		return null;
	},
	
	// -----------------------------------------------
	
	__sentinel: null
});

})();