/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.client.internal.WorkItemChangeRunnable"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.SaveRunnable"); //$NON-NLS-1$

dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory"); //$NON-NLS-1$

dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

(function() {
	
var WorkItemClient					= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var ServiceResponseHandler			= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemProxyFactory			= com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory;
	
dojo.declare("com.ibm.team.apt.web.client.internal.WorkItemChangeRunnable", //$NON-NLS-1$
			 [com.ibm.team.apt.web.client.internal.SaveRunnable], {
	
	_planItem: null,
	_attributeIdentifiers: null,
	_attributeValues: null,
	_updateLinks: null,
	_workflowActionId: null,
	_workflowActionResultStateId: null,
	_undoRunnable: null,
	
	constructor: function(args) {
		dojo.mixin(this, args);
	},
	
	canMergeWith: function(runnable) {
		if (!runnable instanceof com.ibm.team.apt.web.client.internal.WorkItemChangeRunnable)
			return false;
		
		return this._planItem == runnable._planItem;
	},
	
	mergeWith: function(runnable) {
		//merge undo
		var firstUndoRunnable= this._undoRunnable;
		var secondUndoRunnable= runnable._undoRunnable;
		this._undoRunnable= function() {
			secondUndoRunnable();
			firstUndoRunnable();
		};
		
		//merge attributes
		for ( var i = 0; i < this._attributeIdentifiers.length; i++) {
			var id= this._attributeIdentifiers[i];
			var index= dojo.indexOf(runnable._attributeIdentifiers, id);
			if (index >= 0) {
				this._attributeValues[i]= runnable._attributeValues[index];
				
				runnable._attributeIdentifiers.splice(index, 1);
				runnable._attributeValues.splice(index, 1);
			}
		}
		
		this._attributeIdentifiers.push.apply(this._attributeIdentifiers, runnable._attributeIdentifiers);
		this._attributeValues.push.apply(this._attributeValues, runnable._attributeValues);
		
		//merge workflow action
		if (this._workflowActionId == null || runnable._workflowActionId != null) {
			this._workflowActionId= runnable._workflowActionId;
			this._workflowActionResultStateId= runnable._workflowActionResultStateId;
		}
		
		//merge update links
		this._updateLinks.push.apply(this._updateLinks, runnable._updateLinks);
	},
	
	undo: function() {
		this._undoRunnable();
	},

	save: function(successCallback, failureCallback) {
		var args= {
				itemId: this._planItem.getUuid(),
				stateId: this._planItem.__workItem.stateId,
				id: this._planItem.getId()
		};
		if (this._workflowActionId != null) {
			args.action= this._workflowActionId;
		}

		args.updateLinks= this._updateLinks;
		args.attributeIdentifiers= this._attributeIdentifiers;
		args.attributeValues= this._attributeValues;
		
		var serviceObj= {
				self: this,

				success: function(workItem) {
					var planItem= this.self._planItem;
					planItem.__workItem.stateId= workItem.stateId;
					
					if (this.self._workflowActionId != null) {
				  		planItem.getPlanModel().compoundChange(function(builder) {
				  			var newStateId= this.self._workflowActionResultStateId;
				  			
				  			var stateAttribute= planItem.getPlanModel().findAttribute(com.ibm.team.apt.client.PlanItem.STATE);
				  			var newState= stateAttribute._convertToPlanItem(newStateId, planItem);
				  			stateAttribute.setValue(planItem, newState);
				  			
				  			planItem.setWorkflowAction(null);
				  		}, this);
					}
					
					var workingCopy= WorkItemProxyFactory.getWorkItemProxy({id: planItem.getId(), createIfNeeded: false});
					if (workingCopy) {
						workingCopy.fetchWorkItem({
							onSuccess: function() {}, 
							onError: function() {}
						});
					}
			  		
					successCallback();
				},
				failure : function(operationReport) {
					this.self._undoRunnable();
					failureCallback(operationReport);
				}
		};
		var responseHandler= new ServiceResponseHandler(serviceObj, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
		WorkItemClient.postWorkItem(responseHandler, args);
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.client.internal.WorkItemChangeRunnable.constructorArgs= function(planItem, undoRunnable, attributeIdentifiers, attributeValues, updateLinks, workflowActionId, workflowActionResultStateId) {
	return {
		_planItem: planItem,
		_attributeIdentifiers: attributeIdentifiers ? attributeIdentifiers : [],
		_attributeValues: attributeValues ? attributeValues : [],
		_updateLinks: updateLinks ? updateLinks : [],
		_workflowActionId: workflowActionId ? workflowActionId : null,
		_workflowActionResultStateId: workflowActionResultStateId ? workflowActionResultStateId : null,
		_undoRunnable: undoRunnable
	};
};

})();