/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.util.html");
dojo.require("jazz.app.ResourceLink");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var HTML = jazz.util.html;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var ResourceLink = jazz.app.ResourceLink;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/AllTeamAreas.html"),
	
	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.allTeamsProj = messages.allTeamsProj;
	},
	postCreate: function() {
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter), enabled: false});
		this._filterArea.appendChild(this._filterBox.domNode);
		this.uriRoot = (net.jazz.ajax._dataServerRoot || Util.getContextPath(true)) + "/process/project-areas/" + getCurrentProjectArea().itemId;
		this._projectLink = new ResourceLink({
			uri: this.uriRoot,
			presentationUri: "#action=com.ibm.team.dashboard.viewDashboard",
			label: HTML.escape(getCurrentProjectArea().name),
			retainLinkText: true
		}, this._projectLink);
		var serviceObject = {
			self: this,
			success: function(result) {
				if (result && result.children && result.children.length > 0) {
					this.self._areas = result.children;
				}
				this.self._populate();
			},
			failure: function(error) {
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		ProcessClient.getProjectHierarchy({uuid: getCurrentProjectArea().itemId}, srh);
	},
	
	_populate: function() {
		if (!this._areas) {
			this._createRow(messages.allTeamsNone);
		} else {
			this._sortAreas(this._areas);
			this._parseAreas(this._areas, 0, []);
		}
		this._filterBox.setEnabled(true);
	},
	
	_parseAreas: function(areas, indent, pathSegs) {
		for (var i = 0; i < areas.length; i++) {
			var area = areas[i];
			var thisPath = dojo.clone(pathSegs);
			thisPath.push(area.name);
			area.node = this._createRow(area, DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, thisPath), indent);
			area.name = area.name.toLowerCase();
			if (area.children)
				this._parseAreas(area.children, indent+1, thisPath);
		}
	},
	
	_createRow: function(area, href, indent) {
		var result = document.createElement("div");
		result.className = "resultRow";
		if (indent) {
			result.style.paddingLeft = (1 + (indent * 2)) + "em";
		} else {
			result.style.paddingLeft = "1em";
		}
		if (href) {
			var rl = new ResourceLink({
				uri: this.uriRoot + "/team-areas/" + area.itemId,
				presentationUri: href,
				label: HTML.escape(area.name),
				retainLinkText: true
			});
			result.appendChild(rl.domNode);
		} else {
			result.appendChild(document.createTextNode(area.name));
		}
		this._resultsArea.appendChild(result);
		return result;
	},
	
	_doFilter: function(filter) {
		if (this._currentFilter && this._currentFilter == filter)
			return;
		this._currentFilter = filter;
		if (this._areas) {
			if (this._currentFilter.length == 0) 
				this._clearFilter(this._areas);
			else {
				var hits = this._filterAreas(this._areas);
				if (!hits) {
					if (!this._noHitsRow) {
						this._noHitsRow = this._createRow(messages.allTeamsNoneMatch);
					}
					this._noHitsRow.style.display = "block";
				} else if (this._noHitsRow)
					this._noHitsRow.style.display = "none";
			}
		}
	},
	
	_filterAreas: function(areas) {
		var result = false;
		for (var i = 0; i < areas.length; i++) {
			var childHits = false;
			if (areas[i].children && areas[i].children.length > 0)
				childHits = this._filterAreas(areas[i].children);
			if (childHits || areas[i].name.indexOf(this._currentFilter) != -1) {
				areas[i].node.style.display = "block";
				result = true;
			} else {
				areas[i].node.style.display = "none";
			}
		}
		return result;
	},
	
	_clearFilter: function(areas) {
		if (this._noHitsRow)
			this._noHitsRow.style.display = "none";
		for (var i = 0; i < areas.length; i++) {
			areas[i].node.style.display = "block";
			if (areas[i].children && areas[i].children.length > 0)
				this._clearFilter(areas[i].children);
		}
	},
	
	_sortAreas: function(areas) {
		areas.sort(function(a,b) {
			return a.name < b.name ? -1 : 1;
		});
		for (var i = 0; i < areas.length; i++) {
			if (areas[i].children && areas[i].children.length > 0) 
				this._sortAreas(areas[i].children);
		}
	}
	
});

})();
