/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.MembersViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.PagedTableViewlet");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.MembersClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.app.ResourceLink");
dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MembersMessages");

(function() {

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var HoverPopup = jazz.ui.HoverPopup;
var ResourceLink = jazz.app.ResourceLink;
var HTML = jazz.util.html;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var MemberInfo = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo;
var MemberInfoClient = com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient;	
var MembersClient = com.ibm.team.dashboard.viewlets.web.client.internal.MembersClient;
var PagedTableViewlet = com.ibm.team.dashboard.web.ui.PagedTableViewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MembersMessages");

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.MembersViewlet",
	PagedTableViewlet, {
	
	contentClass: "com-ibm-team-dashboard-viewlets-web-ui-internal-MembersViewlet",

	init: function() {
		this._updateTitle();
		this._widgets = new Array();
		this.inherited(arguments);
	},
	
	refresh: function() {
		delete this._totalResults;
		delete this._token;
		this._destroyWidgets();
		this._infoWidgets = new Object();
		this._userItemIds = new Array();
		this.inherited(arguments);
	},
	
	getResultCount: function() {
		return (this._totalResults || this._totalResults == 0) ? this._totalResults : -1;
	},
	
	getNoResultsMessage: function() {
		if (!this.getScopeItem()) {
			return this._createNoScopeMsg();
		}
		if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA)
			return document.createTextNode(messages.noMembersTeam);
		return document.createTextNode(messages.noMembersProject);
	},
	
	getItemsPerPage: function() {
		return this.getPreference("items");
	},
	
	fetch: function(srh, start, count, entries, details){
		// handle Auto(none) scope when no team areas
		if (!this.getScopeItem()) {
			this._totalResults = 0;
			this._totalResultsChanged();
			this.getSite().setSecondaryTitle("");
			srh.doCallback();
			return;
		}
		if (details && !entries) {
			this.fetchDetails(srh, start, count);
			return;
		}
		var serviceObject = {
			self: this,
			success: function(page) {
				if (page && page.totalResults > 0) {
					if (this.self._totalResults != page.totalResults) {
						this.self._totalResults = page.totalResults;
						this.self._totalResultsChanged();
					}
					this.self._token = page.token;
					if (page.members) {
						var members = page.members;
						var result = new Object();
						for (var i = 0; i < members.length; i++) {
							this.self._userItemIds[i+start] = members[i].itemId;
						}
						if (entries)
							result.entries = members;
						if (details)
							result.details = members;
						srh.doCallback(result);
					}
				} else {
					this.self._totalResults = 0;
					this.self._totalResultsChanged();
					srh.doCallback();
				}
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		}
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		var items = this.getPreference("items");
		switch(this.getScope()) {
			case DashboardConstants.SCOPE_TEAM_AREA:
				MembersClient.getMembersForTeamArea(this.getScopeItem().itemId, start, count, this._token ? this._token : "", details, srh2, this.getServiceTracker());
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				MembersClient.getMembersForProjectArea(this.getScopeItem().itemId, start, count, this._token ? this._token : "", details, srh2, this.getServiceTracker());
				break;
		}
	},
	
	fetchDetails: function(srh, start, count) {
		var membersToFetch = new Array();
		var result = new Object();
		result.details = new Array();
		for (var i = start; i < start + count && this._userItemIds[i]; i++) {
			membersToFetch.push(this._userItemIds[i]);
		}
		var serviceObject = {
			self: this,
			success: function(dtos) {
				var j = 0;
				for (var i = 0; i < dtos.length; i++) {
					while (result.details[j]) j++;
					result.details[j] = {details: dtos[i], itemId: this.self._userItemIds[j+start]};
				}
				srh.doCallback(result);
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		}
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		MemberInfoClient.getMemberInfo(membersToFetch, srh2, this.getServiceTracker());
	},

	/* Node[] */ renderEntry: function(member, memberDetails, /* boolean */ zoomed) {
		var columns = new Array();
		var nameCell = document.createElement("td");
		nameCell.className = "nameCell";
		var href = URLUtility.getContributorHref(member.itemId);
		var node = document.createElement("a");
		node.appendChild(document.createTextNode(member.name));
		if (this.isZoomed()) {
			node.href = href;
		} else {
			var rl = new ResourceLink({
				uri: (net.jazz.ajax._dataServerRoot || Util.getContextPath(true)) + "/users/" + encodeURIComponent(member.userId),
				presentationUri: href,
				retainLinkText: true
			}, node);
			this._widgets.push(rl);
		}
		nameCell.appendChild(node);
		columns.push(nameCell);
		
		var roleCell = document.createElement("td");
		roleCell.className = "roleCell";
		for (var i = 0; member.roles && i < member.roles.length; i++) {
			var roleSpan = document.createElement("span");
			roleSpan.appendChild(document.createTextNode(member.roles[i]));
			
			tooltip = new HoverPopup({
				around: roleSpan,
				persist: false,
				createContents: dojo.hitch(this,this._populateRoleTooltip)});
			tooltip.data = member.roles[i];
			this._widgets.push(tooltip);
			
			roleCell.appendChild(roleSpan);
			if (i != member.roles.length - 1)
				roleCell.appendChild(document.createTextNode(", "));
		}
		columns.push(roleCell);
		
		return columns;
	},
	
	/* Node[] */ renderDetails: function(member) {
		if (!this._infoWidgets[member.itemId]) {
			var memberInfo = new MemberInfo({memberInfoDto: member.details, linkTeams: this.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA), linkProjects: this.getSite().canViewDashboard(DashboardConstants.SCOPE_PROJECT_AREA)});
			this._widgets.push(memberInfo);
			this._infoWidgets[member.itemId] = memberInfo;
		}
		return this._createDetailsCell(this._infoWidgets[member.itemId]);
	},
	
	_createDetailsCell: function(infoWidget) {
		var columns = new Array();
		
		var cell = document.createElement("td");
		cell.className = "detailsCell";
		cell.colSpan = 2;
		cell.appendChild(infoWidget.domNode);
		columns.push(cell);
		
		return columns;
	},
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var nodes = I18nUtil.addMessage(editSpan, messages.notScopedEdit)
			this.connect(nodes[0], "onclick", this._handleToggleSettings);
			msgSpan.appendChild(editSpan);
		}
		return msgSpan;
	},
	
	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},
	
	_populateRoleTooltip: function(tip) {
		var div = document.createElement("div");
		div.innerHTML = "Loading...";
		div.className = "com-ibm-team-dashboard-viewlets-web-ui-internal-MembersViewlet-TooltipLoading";
		var serviceObj = {
			node: div,
			success: function(/* string */ description) {
				this.node.className = "com-ibm-team-process-web-viewlets-ui-internal-MembersViewlet-Tooltip";
				if (description)
					this.node.innerHTML = HTML.escape(description);
				else
					this.node.innerHTML = messages.noDescription;
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		MemberInfoClient.getRoleDescription(tip.data, this.getScope(), this.getScopeItem().itemId, srh, this.getServiceTracker());
		return div;
	},
	
	_totalResultsChanged: function() {
		this.updateTotalPageFooter();
		this.getSite().setSecondaryTitle("(" + this._totalResults + ")");
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var updateTitle = newSettings.userTitle != oldSettings.userTitle;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			this.reset(true);
		}
		else if (newSettings.preferences.items != oldSettings.preferences.items) {
			this.reset(false);
		}
		if (updateTitle)
			this._updateTitle();
	},
	
	_updateTitle: function() {
		this.getSite().setTitle((this.getScopeItem() && this.getScopeItem().name) ? dojo.string.substitute(messages.titleScoped, [this.getScopeItem().name]) : messages.title);
	},
	
	onDestroy: function() {
		this._destroyWidgets();
	},
	
	_destroyWidgets: function() {
		while (this._widgets.length > 0)
			this._widgets.pop().destroy();
	}

});

})();
