/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.rtc.web.ui.internal.RTCApplication"); //$NON-NLS-1$

dojo.require("jazz.app._JazzApplication"); //$NON-NLS-1$
dojo.require("com.ibm.team.rtc.web.ui.internal.RepositoryInitializer");
dojo.require("com.ibm.team.rtc.web.ui.internal.ProcessInitializer");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.client.xhr");
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.rtc.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages = dojo.i18n.getLocalization("com.ibm.team.rtc.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
var Util = com.ibm.team.repository.web.ui.internal.Util;

dojo.declare("com.ibm.team.rtc.web.ui.internal.RTCApplication", jazz.app._JazzApplication, { //$NON-NLS-1$

	productName: messages.RationalTeamConcert,
	
	productIconSmall: dojo.moduleUrl("com.ibm.team.rtc.web", "ui/graphics/UICustomizations/RTC_16.png"),
	
	productIconLarge: dojo.moduleUrl("com.ibm.team.rtc.web", "ui/graphics/UICustomizations/RTC_31.png"),
	
	constructor: function(args) {
		com.ibm.team.rtc.web.ui.internal.RepositoryInitializer.init(this, "_onSuccess", "onFailure"); //initialize repo features
	
		this.setName(messages.IBMRationalTeamConcert); //Set default browser title
		this.config.productTitleImg = dojo.moduleUrl("com.ibm.team.rtc.web", "ui/graphics/UICustomizations/Rational-Team-Concert.png");
		this.workbench.registerBodyClickHandler(dojo.hitch(this,this._onResourceClick),1.5); // lower priority than ResourceRegistry
	
		//hardcode jazz.net footer links until branding solution is available.
		if (location.href.indexOf("jazz.net") > 0 && location.href.indexOf("localhost") > 0) {
			this.config.footer = {links: [ 
				{ label: messages.contact, href: "/legal/contact.jsp" },
				{ label: messages.termsOfUse, href: "/legal/terms.jsp" }
			]};
		}
	},
	
	_onSuccess: function() {
		try {
			var contexts = com.ibm.team.rtc.web.ui.internal.ProcessInitializer.init(this); //initialize project areas
			if (contexts && contexts.currentContext) {
				this.config.buttons.push({ //create the home button
					label: "Dashboards",
					iconSrc: this.productIconLarge,
					align: "left",
					pageId: "com.ibm.team.dashboard"
				});
			}						
			if (Util.userIsAdmin()) {
				var curUrl = new dojo._Url(location.href);
				var query = curUrl.query ? "?" + curUrl.query : "";
				var adminLink = {
					label: messages.admin,
					href: dojo.baseUrl + "../../admin" + query,
					align: "right"
				};
				if (this.config.contextSwitcher) {
					adminLink.rightPipe = true;
				}
				this.config.navbarItems.push(adminLink);
			}
		} catch(e) {
			this.onFailure(e);
			return;
		}
		this.start();
	},
	
	/* 
	 * For programatic transitions to resource urls, this function should be called via:
	 * 
	 * jazz.app.currentApplication.handleResource(href)
	 * 
	 * This will trigger the _onResourceClick listener which will ensure that the browser is
	 * redirected to the correct web ui url.
	 */
	handleResource: function(href) {
		if (href) {
			var urlObj = new dojo._Url(href);
			if (!this._onResourceClick(urlObj))
				location.href = href;
		}
	},
	
	/*
	 * Is registered as a body click listener to intercept link clicks for resources. When it sees
	 * a resource link it will prevent the regular browser transition and will fetch the web ui url
	 * from the server before forwarding to the correct destination automatically.
	 */
	_onResourceClick: function(urlObj) {
		// check if this is a local resource link
		var winUrl = new dojo._Url(window.location.href);
		var urlRoot = winUrl.scheme + "://" + winUrl.host + (winUrl.port ? ":" + winUrl.port : "" );
		if (urlObj.uri.indexOf(urlRoot + net.jazz.ajax._contextRoot + "/resource") == -1) {
			return false;
		}
		// send an xhr to the server to see if there is a web ui mapping for this resource
		var deferred = jazz.client.xhrGet({
			url: urlObj.uri
		});
		deferred.addBoth(function(response) {
			try {
				// find the meta refresh tag (if there is one)
				var regex = /<meta\s*http-equiv="refresh".*?url=(\S*)">/;
				var result = response.match(regex);
				if (result.length == 2) {
					// use the DOM to decode the XML entities (if there are any)
					var div = document.createElement("div");
					div.innerHTML = "<a href=\"" + result[1] + "\"></a>";
					var target = div.firstChild.href;
					// preserve debug mode
					if (djConfig.isDebug) {
						var prefix = target;
						var suffix = "";
						var index = target.indexOf("#");
						if (index != -1) {
							prefix = target.substring(0,index);
							suffix = target.substring(index);
						}
						if (prefix.indexOf("?") == -1) {
							prefix = prefix + "?";
						} else {
							prefix = prefix + "&";
						}
						target = prefix + "debug=true" + suffix;
					}
				}
			} catch(e) {
				// fail silently, we will use the resource URL
			}
			if (!target)
				location.href = urlObj.uri;
			else if (!net.jazz.ajax.ui.PlatformUI.getWorkbench().shouldVetoNavigation(target))
				location.href = target;
		});
		return true;
	}
});

})();
