/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletTree");

/*
 * Imports 
 */
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

(function() {

/*
 * Aliases
 */
var ViewletTree = com.ibm.team.dashboard.web.ui.internal.ViewletTree;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletTree",
	dijit.Tree, {
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	
	
	/*
 	 * Private Constructor
 	 * 
 	 * @param Object[] args
 	 * @return <none>
 	 */
	_postCreateConstructor: function() {
		// Handle double-click events on tree nodes
		this.connect(this.domNode, "ondblclick", this._onDoubleClick);
		
		// Constants //------------------------------------------------------------>
		
		this._static._PRE_VIEWLET_ICON_SEGMENT = 
			Util.getContextRoot() + "/service/com.ibm.team.dashboard.common.internal.service.IDashboardImageService";
		
		this._static._DEFAULT_CATEGORY_ICON = 
			dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/icons/folder.gif");
		
		this._static._TYPE_CATEGORY = "category";
		this._static._TYPE_VIEWLET = "viewlet";
		
		// Instance Variables //--------------------------------------------------->
		
		this._selectionListenerCount = 0;
		this._selectionListenerList = new Array();
		
		this._doubleClickListenerCount = 0;
		this._doubleClickListenerList = new Array();
		
		// Expand all top level nodes
		this._expandAllTopLevelNodes();
		
		// IMPORTANT NOTE: Perform operations on nodes from here on because the nodes
		// are lazily created and may not exist when overriding other tree methods		
		
		// Get the node map
		var nodeMap = this._itemNodeMap;
		// Dynamically update the tree icons
		this._updateIcons(nodeMap);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Object message
 	 * 					. dojo.data.Item item
 	 * 					. dijit._TreeNode node
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	_onEnterKey: function(message) {
		this.inherited(arguments);
		// Handle as a double click event
		this.onDoubleClick(message.item);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * 
 	 * @return void
 	 * @overrides super
 	 */
	_expandNode: function(node){
		this.inherited(arguments);
		this._handleEventNodeToggle(node);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * 
 	 * @return void
 	 */
	_handleEventNodeToggle: function(node) {
		// Initialize map tracking first expansion of folders
		if (!this._folderFirstExpandedMap) {
			this._folderFirstExpandedMap = new Array();
		}

		var label = node.label;
		// Check to see if this node has been expanded before
		if ((label in this._folderFirstExpandedMap) &&
			(this._folderFirstExpandedMap[label])) {
			// Nothing to do
			return;		
		}
		// Track which nodes have been expanded already
		this._folderFirstExpandedMap[label] = true;
		// Get the assoicate tree item
		var item = node.item;
		// Get the children
		var children = item.children;
		// Ensure we have an array
		if ((children instanceof Array) == false) {
			return;
		}
		// Get the node map
		var nodeMap = this._itemNodeMap;
		// Process children
		for (var i = 0; i < children.length; i++) {
			var child = children[i];
			// Get the child's name
			var name = new String(child.name);
			// Ensure name is defined and is in the node map
			if (name.length == 0) {
				continue;
			} else if ((name in nodeMap) == false) {
				continue;
			}
			// Get the corresponding node in the map
			var node = nodeMap[name];
			// Update the node's icon
			this._updateNodeIcon(node);
		}
	},	
	
	/*
 	 * Method
 	 * 
 	 * @param Object args
 	 * 			Field: Object self
 	 * 				Method: _selectionChanged(event)
 	 * @return <none>
 	 */
	_addSelectionListener: function(args) {
		// Get the listener
		var listener = args.self;
		// Add the listener to the listener list
		this._selectionListenerList[this._selectionListenerCount] = listener;
		// Increment the listener list count
		this._selectionListenerCount++;
	},		
	
	/*
 	 * Method
 	 * 
 	 * @param Object args
 	 * 			Field: Object self
 	 * 				Method: _doubleClickOccurred(event)
 	 * @return <none>
 	 */
	_addDoubleClickListener: function(args) {
		// Get the listener
		var listener = args.self;
		// Add the listener to the listener list
		this._doubleClickListenerList[this._doubleClickListenerCount] = listener;
		// Increment the listener list count
		this._doubleClickListenerCount++;
	},		
	
	/*
 	 * Public Method
 	 * 
 	 * @param dojo.data item
 	 * @param TreeNode node
 	 * @return void
 	 * @overrides
 	 */	
	onClick: function(item, node) {
		// Notify all listeners
		for (var i = 0; i < this._selectionListenerList.length; i++) {
			this._selectionListenerList[i]._selectionChanged(item);
		}
	},	
			
	/*
 	 * Method
 	 * 
 	 * @param Event event
 	 * @return <none>
 	 * @overrides
 	 */	
	onDoubleClick: function(event) {
		// Notify all listeners
		for (var i = 0; i < this._doubleClickListenerList.length; i++) {
			this._doubleClickListenerList[i]._doubleClickOccurred(event);
		}
	},			

	/*
 	 * Private Method
 	 * 
 	 * @param Object nodeMap
 	 * 
 	 * @return void
 	 */		
	_updateIcons: function(nodeMap) {
		for (var name in nodeMap) {
			// Get the node
			var node = nodeMap[name];
			if (!node) {
				continue;
			}
			// Process the node
			this._updateNodeIcon(node);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * 
 	 * @return void
 	 */		
	_updateNodeIcon: function(node) {
		// Process according to type
		if (!node.item) {
			return;
		} else if ((node.item.type instanceof Array) === false) {
			return;
		} else if (node.item.type[0] === this._static._TYPE_CATEGORY) {
			// Category node
	        this._updateCategoryIcon(node);
	    } else if (node.item.type[0] === this._static._TYPE_VIEWLET) {
			// Viewlet node
			this._updateViewletIcon(node);
	    }
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * 
 	 * @return void
 	 */		
	_updateCategoryIcon: function(node) {
		this._updateIcon(node, this._static._DEFAULT_CATEGORY_ICON);
	},
		
	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * @param String src
 	 * 
 	 * @return void
 	 */		
	_updateIcon: function(node, src) {
		if (!node.iconNode) {
			return;
		}
		node.iconNode.src = src;
	},		
		
	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * 
 	 * @return void
 	 */		
	_updateViewletIcon: function(node) {
		var src;
		if (((node.item.icon instanceof Array) === false) || !node.item.icon[0]) {
			// no icon specified in entry; use definition's icon
			src = this._static._PRE_VIEWLET_ICON_SEGMENT + "?definitionId=" + node.item.id[0];
		} else if (node.item.icon[0].indexOf("://") === -1) {
			// Icon format:
			// com.ibm.team.dashboard.viewlets.service/ui/internal/images/icon-status.gif
			src = this._static._PRE_VIEWLET_ICON_SEGMENT + "?path=" + node.item.icon[0];
		} else {
			// Icon format:
			// https://jazz.net/favicon.ico
			src = node.item.icon[0];
		}
		// Set the icon
		this._updateIcon(node, src);
	},		

	/*
 	 * Method
 	 * 
 	 * @param event - Event
 	 * @return <none>
 	 */		
	_onDoubleClick: function(event) {
		// Translate the current DOM node into a tree node
        var treeNode = dijit.getEnclosingWidget(event.target);
		// Ensure is defined
		if (treeNode == undefined) {
			return;
		} else if (treeNode == null) {
			return;
		}
		// Ensure has item property
		if (("item" in treeNode) == false) {
			return;
		}
		// Get the item currently selected in the tree
		var item = treeNode.item;		
		// Handle as an on double click event
		this.onDoubleClick(item);
	},	
	
	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 * @overrides
 	 */	
	postCreate: function() {
		// Call super
		com.ibm.team.dashboard.web.ui.internal.ViewletTree.superclass.postCreate.call(this);
		// Call our private constructor
		this._postCreateConstructor();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dojo.data.Item item
 	 * @return String
 	 * @overrides super
 	 */	
	getLabelClass: function(item) {
		if (item && 
			item.type && 
			(item.type[0] === this._static._TYPE_VIEWLET)) {
			return "viewlet-entry";
		}
		return "category-entry";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dojo.data.Item item
 	 * @return String
 	 * @overrides super
 	 */	
	getIconClass: function(item){
		return "tree-icon";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Widget newFocusNode
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	_onTreeFocus: function(newFocusNode) {
		var oldFocusNode = this.lastFocused;
		
		// Remove selection from old focus node
		if ((newFocusNode !== oldFocusNode) &&
			oldFocusNode &&
			oldFocusNode.rowNode) {
			dojo.removeClass(oldFocusNode.rowNode, "focus-entry");
		}
		// Add selection to new focus node
		if (newFocusNode &&
			newFocusNode.rowNode) {
			dojo.addClass(newFocusNode.rowNode, "focus-entry");
			// Handle as a click event
			this.onClick(newFocusNode.item, newFocusNode);
		}
		
		// Call super
		this.inherited(arguments);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object args
 	 * 
 	 * @return dijit._TreeNode
 	 * @overrides super
 	 */	
	_createTreeNode: function(args) {
		
		var treeNode = this.inherited(arguments);
		
		if (dojo.isSafari) {
			// Safari on Windows does not support onfocus and onblur events
			// http://www.quirksmode.org/dom/events/blurfocus.html
			// Emulate onfocus events with onclick events
			treeNode.connect(treeNode.domNode, "onclick", treeNode._onNodeFocus);
		}

		return treeNode;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_expandAllTopLevelNodes: function() {
		// Ensure we have a root node
		if (!this.rootNode) {
			return;
		}
		// Get all of the top level nodes
		var treeNodeList = this.rootNode.getChildren();
		// Ensure we have an array
		if ((treeNodeList instanceof Array) === false) {
			return;
		}
		// Expand each top level node
		for (var index = 0; index < treeNodeList.length; index++) {
			var treeNode = treeNodeList[index];
			// Ensure the node is a folder
			if (!treeNode) {
				continue;
			} else if (!treeNode.isExpandable) {
				continue;
			}
			// Track which nodes have been expanded already
			this._folderFirstExpandedMap[treeNode.label] = true;
			// Expand the node
			this._expandNode(treeNode);
		}
	}	
	
});

})();
