/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor"); //$NON-NLS-1$
	dojo.require("jazz.core.RegistryFactory"); //$NON-NLS-1$
	dojo.require("jazz.core.loader"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AccessControlEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TimelineEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
	dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
	dojo.require("jazz.client.xhr"); //$NON-NLS-1$
	dojo.require("jazz.util.error"); //$NON-NLS-1$
	dojo.require("dojo.date"); //$NON-NLS-1$
	dojo.require("dojo.date.locale"); //$NON-NLS-1$
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var RegistryFactory = jazz.core.RegistryFactory;
	var ON_CLICK = "onclick"; //$NON-NLS-1$
	var TEAM_AREA_EDITOR = "com.ibm.team.process.editTeamArea"; //$NON-NLS-1$
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor", com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor, { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ProjectAreaEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		initialize: function(args){
			this.projectAreaItemId = args.itemId;
			this.context = ProcessConstants.EditorContext.PROJECT;
			this._projectAreaIconSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/projectAreaIcon.png"); //$NON-NLS-1$ //$NON-NLS-2$
			this.allProcessDefinitions = [];
			
			this.accessControl = {};
			this.accessControl.publicVisible = true;
			this.accessControl.memberVisible = false;
			this.accessControl.specificUserVisible = false;
			
			this.projectAreaLinks = [];
			this.existingProjectAreaLinks = [];
			this.newProjectAreaLinks = [];
			this.removedProjectAreaLinks = [];
		},
		
		postCreate: function(){
			this.inherited(arguments);
			this._createTableHeader();
		},
		
		toggleViewEditMode: function(mode){
			this.inherited(arguments);
		},
		
		createCustomTabs: function(){
			this.createTab(this.messages.ProjectAreaEditor_TimelineTabLabel, ProcessConstants.TabEnum.TIMELINE, this._createTimelineEditor, ProcessConstants.TabEnum.ROLES);
			this.createTab(this.messages.ProjectAreaEditor_AccessControlTabLabel, ProcessConstants.TabEnum.ACCESS_CONTROL, this._createAccessControl);
			
			this._loadRegistryTabs();
			
		},
		
		_loadRegistryTabs: function(){
			// Get extension registry
			var registry = RegistryFactory.getRegistry();
			if (!registry) {
				return;
			}
			// Get extension points
			var extensionsPoints = registry.getExtensionPoint(ProcessConstants.ProjectAreaExtensionPage.EXTENSION_POINT_PROFILE_PAGES);
			if (!extensionsPoints) {
				return;
			}
			// Get extensions
			var extensions = extensionsPoints.getExtensions();
			if ((extensions instanceof Array === false) ||
			(extensions.length === 0)) {
				return;
			}
			// Get elements
			for (var index = 0; index < extensions.length; index++) {
				var elements = extensions[index].getConfigurationElements();
				if ((elements instanceof Array === false) ||
				(elements.length === 0)) {
					continue;
				}
				this._processTabElements(elements);
			}
		},
		
		/*
		 * @param Array elements
		 *
		 * @return void
		 */
		_processTabElements: function(elements){
		
			for (var index = 0; index < elements.length; index++) {
				var element = elements[index];
				if (element.getName() !== ProcessConstants.ProjectAreaExtensionPage.ELEMENT_PAGE) {
					continue;
				}
				// Get id
				var id = element.getAttribute(ProcessConstants.ProjectAreaExtensionPage.ATTRIBUTE_ID);
				if (!id) {
					continue;
				}
				// Get title
				var title = element.getAttribute(ProcessConstants.ProjectAreaExtensionPage.ATTRIBUTE_TITLE);
				if (!title) {
					continue;
				}
				// Get widget
				var widget = element.getAttribute(ProcessConstants.ProjectAreaExtensionPage.ATTRIBUTE_WIDGET);
				if (!widget) {
					continue;
				}
				
				this.createTab(title, id, undefined, undefined, widget);
			}
			
		},
		
		customSave: function(saveObj){
		
			// TODO Don't like this...this is really the job of the access control
			// tab to handle but since it doesn't exist until after we create 
			// the project have to initialize here instead.	
			if (this.isNew) {
				saveObj.publicVisible = true;
				saveObj.memberVisible = false;
				saveObj.specificUserVisible = false;
			}
			
			// Workitem 74324
			// Don't like this either...this is really the job of the access control
			// tab to handle but since the save participant is not added until the
			// the tab is actually clicked on...we have to initialize here.
			if (saveObj.publicVisible === undefined) {
				saveObj.publicVisible = this.accessControl.publicVisible;
				saveObj.memberVisible = this.accessControl.memberVisible;
				saveObj.specificUserVisible = this.accessControl.specificUserVisible;
			}
			
			if (this.linksHaveChanged()) {
				this._postProjectAreaLinks(saveObj);
				if (!this.isNew) { //cannot clear until we have a chance to post
					// the back link with the UUID of the new project area
					this.newProjectAreaLinks = [];
					this.removedProjectAreaLinks = [];
				}
			}
		},
		
		_postProjectAreaLinks: function(saveObj){
			var projectAreasLinksUrl = this._getProjectAreaLinksURL();
			if (this.newProjectAreaLinks.length > 0 || this.removedProjectAreaLinks.length > 0) {
				var jsonLinks = {};
			}
			if (this.newProjectAreaLinks.length > 0) {
				var addedLinks = [];
				jsonLinks.add = addedLinks;
			}
			if (this.removedProjectAreaLinks.length > 0) {
				var removedLinks = [];
				jsonLinks.remove = removedLinks;
			}
			var createTrace = this.messages.ProjectAreaEditor_CreatingBackLinks;
			dojo.forEach(this.newProjectAreaLinks, function(projectAreaLink){
				var projectAreaInfo = projectAreaLink.projectAreaInfo;
				var newLink = {};
				newLink.label = projectAreaInfo.label;
				newLink.linkType = projectAreaLink.linkType;
				newLink.targetUrl = projectAreaInfo.targetUrl;
				newLink.targetServicesUrl = projectAreaInfo.servicesUrl;
				if (projectAreaInfo.consumerRegistryUrl) {
					newLink.consumerRegistryUrl = projectAreaInfo.consumerRegistryUrl;
				}
				
				addedLinks.push(newLink);
				
				var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
				if (backLinkConsumerRegistryUrl && !this.isNew) {
					createTrace = "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_CreatingBackLinkOfType, [projectAreaLink.linkType, projectAreaInfo.label]); //$NON-NLS-1$
					this.postBackLink(projectAreaLink, createTrace);
				}
			}, this);
			
			var deleteTrace = this.messages.ProjectAreaEditor_DeletingBackLinks;
			dojo.forEach(this.removedProjectAreaLinks, function(projectAreaLink){
				removedLinks.push(projectAreaLink.linkUrl);
				
				//delete the back link
				var targetUrl = this._getThisTargetUrl();
				var targetConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
				if (targetConsumerRegistryUrl) {
					var linkType = this._getPeerLinkType(projectAreaLink.linkType);
					var deleteBackLinkUrl = targetConsumerRegistryUrl + "?type=" + linkType + "&resource=" + targetUrl; //$NON-NLS-1$ //$NON-NLS-2$
					deleteTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_DeletingBackLinkOfType, [linkType, projectAreaLink.projectAreaInfo.label]); //$NON-NLS-1$
					var deferred = jazz.client.xhrDelete({
						url: deleteBackLinkUrl,
						handleAs: "xml", //$NON-NLS-1$
						load: dojo.hitch(this, function(){
							deleteTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_DeletedBackLink, [linkType, projectAreaLink.projectAreaInfo.label]); //$NON-NLS-1$
							this.handleServerResponseSuccess();
						}),
						error: dojo.hitch(this, function(error){
							var listing = projectAreaLink.projectAreaInfo.label + "," + linkType; //$NON-NLS-1$
							deleteTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorDeletingBackLinkWithReason, [listing]); //$NON-NLS-1$
							this._configureErrorData(error, deleteTrace);
							this._handleDeleteProjectAreaLinkWarning(deferred, error);
						})
					});
				}
			}, this);
			saveObj.jsonLinks = dojo.toJson(jsonLinks);
		},
		
		postBackLink: function(projectAreaLink, createTrace){
			var index = window.location.hash.toLowerCase().indexOf("testdiscovery=true"); //$NON-NLS-1$
			var rootServicesXml;
			if (index > -1) {
				rootServicesXml = "processtest/rootservices.xml"; //$NON-NLS-1$
			} else {
				rootServicesXml = ProcessConstants.ROOTSERVICESXML;
			}
			this._postBackLink(projectAreaLink, createTrace, rootServicesXml);
		},
		
		_postBackLink: function(projectAreaLink, createTrace, rootServicesXml){
			createTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_ReadingRootservices, [rootServicesXml]); //$NON-NLS-1$
			var deferred = jazz.client.xhrGet({
				url: rootServicesXml,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, function(servicesXml){
					this._postBackLink2(servicesXml, projectAreaLink, createTrace);
					return servicesXml;
				}),
				error: dojo.hitch(this, function(error){
					createTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_FailedRootServicesResource, [rootServicesXml]); //$NON-NLS-1$
					this._configureErrorData(error, createTrace);
					this._handlePostProjectAreasLinkWarning(deferred, error);
					return error;
				})
			});
		},
		
		/*
		 * Parse the rootservices xml to find the service provider catalog for
		 * the link type we are about to post
		 */
		_postBackLink2: function(servicesXml, projectAreaLink, createTrace){
			var children = servicesXml.childNodes;
			var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var peerLinkType = this._getPeerLinkType(projectAreaLink.linkType);
			var tagName = UIHelper.getTagNameForLinkType(peerLinkType);
			var namespace = UIHelper.getNamespaceForLinkType(peerLinkType);
			var descriptionNodeChildren = descriptionNode.childNodes;
			var linkServiceProvider = UIHelper.getNode(descriptionNodeChildren, tagName, namespace);
			if (!linkServiceProvider) {
				this._useFrontingServerRootServicesResource(descriptionNodeChildren, projectAreaLink, createTrace, peerLinkType);
				return;
			}
			
			var serviceProviderResourceNode = UIHelper.getAttributeNode(linkServiceProvider, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var serviceProviderResource = serviceProviderResourceNode.value;
			createTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_DeterminingProjectAreaServicesURL, [serviceProviderResource]); //$NON-NLS-1$
			var deferred = jazz.client.xhrGet({
				url: serviceProviderResource,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, function(serviceProviderCatalogXml){
					this._postBackLink3(serviceProviderCatalogXml, projectAreaLink, createTrace);
				}),
				error: dojo.hitch(this, function(error){
					createTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_FailedServiceProviderResource, [serviceProviderResource]); //$NON-NLS-1$
					this._configureErrorData(error, createTrace);
				})
			});
		},
		
		_useFrontingServerRootServicesResource: function(descriptionNodeChildren, projectAreaLink, createTrace, peerLinkType){
			var friendsNode = UIHelper.getNode(descriptionNodeChildren, "friends", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$ 
			var friendsResourceNode = UIHelper.getAttributeNode(friendsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
			var friendsResourceURL = friendsResourceNode.value;
			var frontingServerRootServicesResource;
			var deferred = jazz.client.xhrGet({
				url: friendsResourceURL,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, function(friendsListingXml){
					if (!friendsListingXml) {
						return friendsListingXml;
					}
					
					var children = friendsListingXml.childNodes;
					var rdfNode = UIHelper.getNode(children, "RDF", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
					var allFriends = UIHelper.getNodes(rdfNode.childNodes, "Friend", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
					var specialNode;
					if (allFriends.length > 0) {
						dojo.forEach(allFriends, function(friend){
							if (!specialNode) {
								var friendChildren = friend.childNodes;
								specialNode = UIHelper.getNode(friendChildren, "internal", ProcessConstants.NAMESPACE_JFS); //$NON-NLS-1$
								if (specialNode && specialNode.firstChild && specialNode.firstChild.data === "true") { //$NON-NLS-1$
									var rootServicesNode = UIHelper.getNode(friendChildren, "rootServices", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
									var rootServicesResourceNode = UIHelper.getAttributeNode(rootServicesNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
									frontingServerRootServicesResource = rootServicesResourceNode.value;
									this._postBackLink(projectAreaLink, createTrace, frontingServerRootServicesResource);
								} else {
									specialNode = undefined; // internal but false
								}
							}
						}, this);
					}
					if (!specialNode) {
						var error = {};
						error.message = dojo.string.substitute(this.messages.ProjectAreaEditor_NoLinkServiceProvder, [peerLinkType]);
						this._handlePostProjectAreasLinkWarning(null, error);
					}
					return friendsListingXml;
				}),
				error: dojo.hitch(this, function(error){
					return error;
				})
			});
		},
		
		/*
		 * Parse the service provider catalog xml to find the service provider
		 * that matches the project area that is being edited so we can populate
		 * the servicesUrl with the correct resource for the post of a back link
		 */
		_postBackLink3: function(serviceProviderCatalogXml, projectAreaLink, createTrace){
			if (!serviceProviderCatalogXml) {
				this._handlePostProjectAreasLinkWarning();
				return;
			}
			
			var children = serviceProviderCatalogXml.childNodes;
			var serviceProviderCatalog = UIHelper.getNode(children, "ServiceProviderCatalog", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			children = serviceProviderCatalog.childNodes;
			var entries = UIHelper.getNodes(children, "entry", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
			var providers = [];
			dojo.forEach(entries, function(entry){
				var serviceProvider = UIHelper.getNode(entry.childNodes, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				providers.push(serviceProvider);
			}, this);
			
			var servicesUrl;
			var consumerRegistryUrl;
			dojo.forEach(providers, function(serviceProvider){
				if (!servicesUrl) {
					var serviceProviderChildren = serviceProvider.childNodes;
					var detailsNode = UIHelper.getNode(serviceProviderChildren, "details", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
					var attributeNode = UIHelper.getAttributeNode(detailsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
					var url = attributeNode.value;
					var fullPath = url.split("/"); //$NON-NLS-1$
					if (fullPath.length > 0) {
						if (fullPath.pop() == this.projectAreaItemId) {
							var servicesNode = UIHelper.getNode(serviceProviderChildren, "services", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
							attributeNode = UIHelper.getAttributeNode(servicesNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
							servicesUrl = attributeNode.value;
							var consumerRegistryNode = UIHelper.getNode(serviceProviderChildren, "consumerRegistry", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
							if (consumerRegistryNode) {
								attributeNode = UIHelper.getAttributeNode(consumerRegistryNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
								consumerRegistryUrl = attributeNode.value;
							}
						}
					}
				}
			}, this);
			
			if (!servicesUrl) {
				var error = {};
				error.message = this.messages.ProjectAreaEditor_NoMatchingProjectAreaForLink;
				this._handlePostProjectAreasLinkWarning(null, error);
				return;
			}
			var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
			var deferred = jazz.client.rawXhrPost({
				url: backLinkConsumerRegistryUrl,
				contentType: "application/xml; charset=utf-8", //$NON-NLS-1$
				handleAs: "xml", //$NON-NLS-1$
				postData: this._getProjectAreaBackLinkPostData(projectAreaLink, servicesUrl, consumerRegistryUrl),
				load: dojo.hitch(this, this.handleServerResponseSuccess),
				error: dojo.hitch(this, function(error){
					createTrace += "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_FailedToPostBackLink, [projectAreaLink.linkType, backLinkConsumerRegistryUrl]); //$NON-NLS-1$
					this._configureErrorData(error, createTrace);
					this._handlePostProjectAreasLinkWarning(deferred, error);
				})
			});
		},
		
		_configureErrorData: function(error, trace){
			error.errorData = {};
			error.errorData.identifier = "com.ibm.team.process.projectLinks"; //$NON-NLS-1$
			error.errorData.internalReportInfos = [];
			error.errorData.internalReportInfos[0] = {};
			error.errorData.internalReportInfos[0].identifier = "com.ibm.team.process.internal.linksException"; //$NON-NLS-1$
			error.errorData.internalReportInfos[0].data = trace;
		},
		
		_handleDeleteProjectAreaLinkWarning: function(deferred, error){
			if (error.message) {
				var errorHeader = this.messages.ProjectAreaEditor_ErrorDeletingBackLink;
				error.message = decodeURIComponent(jazz.util.error.getDefaultXhrErrMsg(deferred));
			} else if (error.responseText) {
				errorHeader = dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorDeletingBackLinkWithReason, [error.responseText]);
			} else {
				var message = error.message;
				error.message = error.trace;
				error.message += ProcessConstants.BR + message;
				errorHeader = this.messages.ProjectAreaEditor_ErrorDeletingBackLink;
			}
			this.handleServerResponseWarning(error, errorHeader);
		},
		
		_getBackLinkConsumerRegistryUrl: function(projectAreaLink){
			return projectAreaLink.projectAreaInfo.consumerRegistryUrl;
		},
		
		_getProjectAreaBackLinkPostData: function(projectAreaLink, servicesUrl, consumerRegistryUrl){
			var label = this._toXmlEncoded(this.name.value);
			var linkType = this._getPeerLinkType(projectAreaLink.linkType);
			var targetUrl = this._getThisTargetUrl();
			return this._getPostData(linkType, label, targetUrl, servicesUrl, consumerRegistryUrl);
		},
		
		_toXmlEncoded: function(plainText){
			var xmlText = new String(plainText);
			// do & case first to avoid double substitutions
			xmlText = xmlText.replace(/&/g, "&amp;"); //$NON-NLS-1$ 
			xmlText = xmlText.replace(/\"/g, "&quot;"); //$NON-NLS-1$
			// skip &apos;
			xmlText = xmlText.replace(/</g, "&lt;"); //$NON-NLS-1$
			xmlText = xmlText.replace(/>/g, "&gt;"); //$NON-NLS-1$
			return xmlText;
		},
		
		_getPostData: function(linkType, label, targetUrl, servicesUrl, consumerRegistryUrl){
			var postData = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + //$NON-NLS-1$
			"<jp:link \n" + //$NON-NLS-1$
			"xmlns:rdf=\"" + //$NON-NLS-1$
			ProcessConstants.NAMESPACE_RDF +
			"\"\n" + //$NON-NLS-1$
			"xmlns:jp=\"" + //$NON-NLS-1$
			ProcessConstants.NAMESPACE_JP +
			"\"\n" + //$NON-NLS-1$
			"xmlns:dc=\"" + //$NON-NLS-1$
			ProcessConstants.NAMESPACE_DC +
			"\"\n" + //$NON-NLS-1$ 
			"xmlns:oslc=\"" + //$NON-NLS-1$
			ProcessConstants.NAMESPACE_OSLC +
			"\">\n" + //$NON-NLS-1$
			"<jp:link-type>" + //$NON-NLS-1$
			linkType +
			"</jp:link-type>\n" + //$NON-NLS-1$
			"<oslc:ServiceProvider>\n" + //$NON-NLS-1$
			"<dc:title>" + //$NON-NLS-1$
			label +
			"</dc:title>\n" + //$NON-NLS-1$
			"<oslc:details rdf:resource=\"" + //$NON-NLS-1$
			targetUrl +
			"\"/>\n" + //$NON-NLS-1$
			"<oslc:services rdf:resource=\"" + //$NON-NLS-1$
			servicesUrl +
			"\"/>\n"; //$NON-NLS-1$
			if (consumerRegistryUrl) {
				postData += "<jp:consumerRegistry rdf:resource=\"" + //$NON-NLS-1$
				consumerRegistryUrl +
				"\"/>\n"; //$NON-NLS-1$	
			}
			
			postData += "</oslc:ServiceProvider>\n" + //$NON-NLS-1$
			"</jp:link>"; //$NON-NLS-1$
			return postData;
		},
		
		//this needs to be keep in synch with the link types in AddProjectLinkDialog.js
		//until such time as there is a mechanism to discover the available link types.
		_getPeerLinkType: function(linkType){
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return ProcessConstants.LinkTypes.IMPLEMENTED_BY;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return ProcessConstants.LinkTypes.IMPLEMENTS;
				case ProcessConstants.LinkTypes.VALIDATES:
					return ProcessConstants.LinkTypes.VALIDATED_BY;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return ProcessConstants.LinkTypes.VALIDATES;
				case ProcessConstants.LinkTypes.TESTS:
					return ProcessConstants.LinkTypes.TESTED_BY;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return ProcessConstants.LinkTypes.TESTS;
				case ProcessConstants.LinkTypes.RELATED_TO:
					return ProcessConstants.LinkTypes.RELATED_TO;
			}
			return null;
		},
		
		_getThisTargetUrl: function(){
			var serverAddress = this._getServerUrl();
			return serverAddress + net.jazz.ajax._contextRoot + "/process/project-areas/" + this.projectAreaItemId; //$NON-NLS-1$
		},
		
		_handlePostProjectAreasLinkWarning: function(deferred, error){
			if (error && error.message) {
				var errorHeader = this.messages.ProjectAreaEditor_ErrorCreatingBackLink;
				if (deferred) {
					error.message = decodeURIComponent(jazz.util.error.getDefaultXhrErrMsg(deferred));
				}
			} else if (error && error.responseText) {
				errorHeader = dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorCreatingBackLinkWithReason, [error.responseText]);
			} else {
				errorHeader = this.messages.ProjectAreaEditor_ErrorCreatingBackLink;
			}
			this.handleServerResponseWarning(error, errorHeader);
		},
		
		_createAccessControl: function(){
		
			var editor = new com.ibm.team.process.web.ui.internal.admin.projects.AccessControlEditor({
				editor: this
			});
			
			this.accessControlEditor = editor;
			this.twoColumnMainColumn.appendChild(editor.domNode);
			return editor;
			
			
		},
		
		_createTimelineEditor: function(){
		
			var editor = new com.ibm.team.process.web.ui.internal.admin.projects.TimelineEditor({
				editor: this
			});
			
			this.timelineEditor = editor;
			this.twoColumnMainColumn.appendChild(editor.domNode);
			editor.startup();
			return editor;
			
			
		},
		
		
		getPrimaryFieldLabel: function(){
			return this.messages.ProjectAreaEditor_7;
		},
		
		getFetchErrorMessage: function(error){
			if (error.status == ProcessConstants.SC_FORBIDDEN) {
				return this.messages.ProjectAreaEditor_ErrorNoReadAccess;
			} else {
				return this.messages.ProjectAreaEditor_6;
			}
		},
		
		populateProcessAreaData: function(data){
			this.inherited(arguments);
			this.processLocale = data.locale;
			
			this.accessControl = {};
			this.accessControl.publicVisible = data.publicVisible;
			this.accessControl.memberVisible = data.memberVisible;
			this.accessControl.specificUserVisible = data.specificUserVisible;
			
			// TODO I don't like this....need to load the data
			// independently on that tab otherwise the timing is off
			// old data shows up from the project since this async
			// method may not have ran by the time saved() has been called.
			if (this.accessControlEditor) {
				this.accessControlEditor.refresh();
			}
			// TODO Same as above...need to share the same data otherwise
			// have a potential timing issue in saved().
			if (this.timelineEditor) {
				this.timelineEditor.refresh();
			}
			if (this.permissionEditor) {
				this.permissionEditor.refresh(); //update the roles
			}
			
			this.populateProcessDescription(data.processDTO);
		},
		
		clearState: function(refreshClicked){
			this.inherited(arguments);
			dojox.data.dom.removeChildren(this.teamAreaHierarchyContent);
			this.projectAreaLinks = [];
			this.newProjectAreaLinks = [];
			this.removedProjectAreaLinks = [];
			this.existingProjectAreaLinks = [];
		},
		
		getSaveErrorMessage: function(err){
			return this.messages.ProjectAreaEditor_31;
		},
		
		_disableCheckBox: function(checkbox, span, disable){
			checkbox.disabled = disable;
			dojo.toggleClass(span, "disabled", disable); //$NON-NLS-1$
		},
		
		populateFilteredProcesses: function(e){
			this.localeSelector.disabled = this.localeFilterToggle.disabled || !this.localeFilterToggle.checked;
			
			var definitionsToShow;
			
			if (this.localeFilterToggle.checked) {
				var locale = this.localeSelector.options[this.localeSelector.selectedIndex].value;
				definitionsToShow = dojo.filter(this.allProcessDefinitions, function(definition){
					return locale === definition.locale;
				});
			} else {
				definitionsToShow = this.allProcessDefinitions;
			}
			
			this.processSelector.options.length = 0;
			
			var newOptions = dojo.map(definitionsToShow, function(definition){
				var option = new Option(this.getTranslatedProcessDefinitionName(definition), definition.itemId);
				option.translatedSummary = this.getTranslatedProcessDefinitionSummary(definition);
				option.locale = definition.locale;
				option.displayableLocale = definition.displayableLocale;
				return option;
			}, this);
			
			newOptions.sort(function(a, b){
				return a.text.localeCompare(b.text);
			});
			
			dojo.forEach(newOptions, function(option){
				if (dojo.isIE) {
					this.processSelector.options[this.processSelector.options.length] = option;
				} else {
					this.processSelector.appendChild(option);
				}
			}, this);
			
			this.processSelector.selectedIndex = 0;
			this.displaySelectedProcessInformation();
		},
		
		populateProcessSelector: function(data){
			if (data !== null && data.length > 0) {
				this.processDescription.style.display = ProcessConstants.NONE;
				this.processSelection.style.display = ProcessConstants.BLOCK;
				
				this.allProcessDefinitions.length = 0;
				
				// populate the 'allProcessDefinitions' list
				dojo.forEach(data, function(processDef){
					if (processDef.supportedLocales && processDef.supportedLocales.length > 0) {
						for (var i = 0; i < processDef.supportedLocales.length; i++) {
							var localeDefEntry = dojo.clone(processDef);
							localeDefEntry.locale = processDef.supportedLocales[i];
							localeDefEntry.displayableLocale = processDef.displayableLocales[i];
							this.allProcessDefinitions.push(localeDefEntry);
						}
					} else {
						this.allProcessDefinitions.push(processDef);
					}
				}, this);
				
				// populate the list of supported locales
				this.localeSelector.options.length = 0;
				var localeNames = [];
				var supportedLocales = [];
				dojo.forEach(data, function(processDefinitionDTO){
					if (processDefinitionDTO.supportedLocales) {
						for (var i = 0; i < processDefinitionDTO.supportedLocales.length; i++) {
							var locale = processDefinitionDTO.supportedLocales[i];
							if (dojo.indexOf(localeNames, locale) == -1) {
								localeNames.push(locale);
								supportedLocales.push({
									localeDisplayName: processDefinitionDTO.displayableLocales[i],
									localeName: locale
								});
							}
						}
					}
				}, this);
				
				supportedLocales.sort(function(a, b){
					return a.localeDisplayName.localeCompare(b.localeDisplayName);
				});
				
				dojo.forEach(supportedLocales, function(locale){
					// add the supported locale to locale selector
					var option = new Option(locale.localeDisplayName, locale.localeName);
					if (dojo.isIE) {
						this.localeSelector.options[this.localeSelector.options.length] = option;
					} else {
						this.localeSelector.appendChild(option);
					}
				}, this);
				
				this.connectHandles.push(dojo.connect(this.localeFilterToggle, ON_CLICK, this, this.populateFilteredProcesses)); //$NON-NLS-1$
				this.connectHandles.push(dojo.connect(this.localeSelector, "onchange", this, this.populateFilteredProcesses)); //$NON-NLS-1$
				this.connectHandles.push(dojo.connect(this.processSelector, "onchange", this, this.displaySelectedProcessInformation)); //$NON-NLS-1$
				if (supportedLocales.length > 0) {
					// attempt to select the supported locale closest to the current client locale
					localeNames = dojo.map(supportedLocales, function(elem){
						return elem.localeName;
					});
					var localeIndex = this.getSupportedClientLocale(localeNames);
					
					if (localeIndex == -1) {
						this.localeFilterToggle.checked = false;
						localeIndex = 0;
					} else {
						this.localeFilterToggle.checked = true;
					}
					
					this.localeSelector.selectedIndex = localeIndex;
				} else {
					this.localeFilterToggle.disabled = true;
					this.localeFilterText.style.color = "#808080"; //$NON-NLS-1$
					this.localeSelector.disabled = true;
					
					// work around lack of min-width support
					if (dojo.isIE) {
						this.localeSelector.style.width = "10em"; //$NON-NLS-1$
					}
				}
				
				this.populateFilteredProcesses();
				this.setDirty(true);
				
			} else {
				// no process definitions, so add the "deploy definitions" controls
				dojox.data.dom.removeChildren(this.processDescription);
				this.processDescription.style.display = ProcessConstants.BLOCK;
				this.processSelection.style.display = ProcessConstants.NONE;
				
				this.processDescription.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_13));
				this.processDescription.appendChild(document.createElement(ProcessConstants.BR));
				this.processDescription.appendChild(document.createElement(ProcessConstants.BR));
				var requiredFlag = document.createElement("span"); //$NON-NLS-1$
				requiredFlag.className = "requiredFlag"; //$NON-NLS-1$
				requiredFlag.appendChild(document.createTextNode("* ")); //$NON-NLS-1$
				this.processDescription.appendChild(requiredFlag);
				var link = document.createElement("a"); //$NON-NLS-1$
				link.tabIndex = 2;
				link.href = "#"; //$NON-NLS-1$
				link.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_14));
				this.connect(link, ON_CLICK, "deployPredefinedProcessDefinitions"); //$NON-NLS-1$ //$NON-NLS-2$
				this.processDescription.appendChild(link);
			}
		},
		
		deployPredefinedProcessDefinitions: function(e){
			dojo.stopEvent(e);
			var srh = new ServiceResponseHandler(this, "handleDeployTemplateSuccess", "handleDeployTemplateError"); //$NON-NLS-1$ //$NON-NLS-2$
			this.displayPendingMessage(this.messages.ProjectAreaEditor_15);
			ProcessClient.deployPredefinedProcessDefinitions(srh);
		},
		
		handleDeployTemplateSuccess: function(){
			this.setDirty(this.changesPending());
			this.displaySuccessMessage(this.messages.ProjectAreaEditor_16);
			this.refreshProcessDefs();
		},
		
		handleDeployTemplateError: function(error){
			this.handleServerResponseError(error, this.messages.ProjectAreaEditor_5);
		},
		
		refreshProcessDefs: function(){
			dojox.data.dom.removeChildren(this.processDescription);
			this.processDescription.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_17));
			var srh = new ServiceResponseHandler(this, "populateProcessSelector", "handleFetchProcessError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getPredefinedProcesses(srh);
		},
		
		handleFetchProcessError: function(error){
			this.handleServerResponseError(error, this.messages.ProjectAreaEditor_4);
		},
		
		populateProcessDescription: function(data){
			dojox.data.dom.removeChildren(this.processDescription);
			this.processDescription.style.display = ProcessConstants.BLOCK;
			this.processSelection.style.display = ProcessConstants.NONE;
			
			var empty = true;
			if (data.name) {
				empty = false;
				var div = document.createElement(ProcessConstants.DIV);
				div.style.marginBottom = "1em"; //$NON-NLS-1$
				div.appendChild(document.createTextNode(data.name));
				var hr = document.createElement("hr"); //$NON-NLS-1$
				hr.style.border = ProcessConstants.NONE;
				hr.style.borderTop = "1px solid #EEEEEE"; //$NON-NLS-1$
				div.appendChild(hr);
				this.processDescription.appendChild(div);
			}
			if (data.details) {
				empty = false;
				div = document.createElement(ProcessConstants.DIV);
				div.appendChild(document.createTextNode(data.details));
				this.processDescription.appendChild(div);
			} else if (data.summary) {
				empty = false;
				div = document.createElement(ProcessConstants.DIV);
				div.appendChild(document.createTextNode(data.summary));
				this.processDescription.appendChild(div);
			}
			if (empty) {
				var desc = this.messages.ProjectAreaEditor_NoProcessDescription;
				this.processDescription.appendChild(document.createTextNode(desc));
			}
		},
		
		/**
		 * Check the list of supportedLocales for the closest to the current
		 * client locale. Look for an exact match first, then check for a supported
		 * locale with  the same language code as the current client locale.
		 *
		 * @param supportedLocales
		 *     An array of locale strings like "zh", "en-us", etc.
		 *
		 * @return the index in the specified list of the locale that was found
		 *     or -1 if no match was found
		 */
		getSupportedClientLocale: function(supportedLocales){
			var clientLocale = djConfig.locale;
			
			if (clientLocale) {
				// if the client locale has a direct match, return it
				if (dojo.indexOf(supportedLocales, clientLocale) != -1) {
					return dojo.indexOf(supportedLocales, clientLocale);
				}
				
				// we don't support the client locale directly, so see if we
				// support a locale with the same language code
				if (clientLocale.length > 2) {
					clientLocale = clientLocale.substring(0, 2);
				}
				
				var supportedLocale = -1;
				for (var i = 0; i < supportedLocales.length; i++) {
					if (supportedLocales[i] === clientLocale) {
						return i;
					}
					if (supportedLocales[i].indexOf(clientLocale) === 0) {
						supportedLocale = i;
					}
				}
				return supportedLocale;
			}
		},
		
		getTranslatedProcessDefinitionSummary: function(definition){
			var translatedSummary;
			
			// if locale filtering is on, use the currently selected locale
			// if locale filtering is off, see if the definition entry has an
			// associated locale
			var locale;
			if (this.localeFilterToggle.checked) {
				locale = this.localeSelector.options[this.localeSelector.selectedIndex].value;
			} else {
				locale = definition.locale;
			}
			
			if (locale && definition.supportedLocales && definition.supportedLocales.length > 0) {
				if (dojo.indexOf(definition.supportedLocales, locale) != -1) {
					translatedSummary = definition.translatedSummaries[dojo.indexOf(definition.supportedLocales, locale)];
				}
			}
			
			// try the default summary
			if (!translatedSummary) {
				translatedSummary = definition.defaultSummary;
			}
			
			// default to the untranslated value
			if (!translatedSummary) {
				translatedSummary = definition.summary;
			}
			
			return translatedSummary;
		},
		
		getTranslatedProcessDefinitionName: function(definition){
			var translatedName;
			
			// if locale filtering is on, use the currently selected locale
			// if locale filtering is off, see if the definition entry has an
			// associated locale
			var locale;
			if (this.localeFilterToggle.checked) {
				locale = this.localeSelector.options[this.localeSelector.selectedIndex].value;
			} else {
				locale = definition.locale;
			}
			
			if (locale && definition.supportedLocales && definition.supportedLocales.length > 0) {
				if (dojo.indexOf(definition.supportedLocales, locale) != -1) {
					translatedName = definition.translatedNames[dojo.indexOf(definition.supportedLocales, locale)];
				}
			}
			
			// try the default name
			if (!translatedName) {
				translatedName = definition.defaultName;
			}
			
			// default to the untranslated value
			if (!translatedName) {
				translatedName = definition.name;
			}
			
			return translatedName;
		},
		
		displaySelectedProcessInformation: function(){
			dojox.data.dom.removeChildren(this.processDescription);
			this.processDescription.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_34));
			dojox.data.dom.removeChildren(this.selectedProcessDescription);
			
			var translatedSummary = this.processSelector.options[this.processSelector.selectedIndex].translatedSummary;
			if (translatedSummary) {
				this.selectedProcessDescription.appendChild(document.createTextNode(translatedSummary));
			}
			
			dojox.data.dom.removeChildren(this.selectedProcessLocale);
			var localeString = this.processSelector.options[this.processSelector.selectedIndex].displayableLocale;
			if (!localeString || localeString.length === 0) {
				localeString = this.messages.ProjectAreaEditor_46;
			}
			this.selectedProcessLocale.appendChild(document.createTextNode(localeString));
		},
		
		customRefresh: function(refreshClicked){
			dojox.data.dom.removeChildren(this.processDescription);
			dojox.data.dom.removeChildren(this.linkArea);
			this.projectAreaLinks = [];
			this.processDescription.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_34));
			if (this.isNew) {
				this.name.value = this.messages.ProjectAreaEditor_1;
				var srh = new ServiceResponseHandler(this, "populateProcessSelector", "handleFetchProcessError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getPredefinedProcesses(srh);
			} else {
				srh = new ServiceResponseHandler(this, "populateProcessAreaData", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getProjectAreaByUUID({
					uuid: this.projectAreaItemId
				}, srh);
				
				this.teamAreaHierarchyContent.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_35));
				srh = new ServiceResponseHandler(this, "populateTeamAreaHierarchy", "handleTeamAreaHierarchyFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getProjectHierarchy({
					uuid: this.projectAreaItemId
				}, srh);
				
				this._fetchProjectAreaLinks();
			}
			
			if (refreshClicked) {
				if (this.accessControlEditor) {
					this.accessControlEditor.refresh();
				}
				if (this.timelineEditor) {
					this.timelineEditor.refresh();
				}
				for (tabId in this._tabs) {
					var tab = this._tabs[tabId];
					if (this.isActivatedExtensionTab(tab)) {
						tab.mainPage.refresh()
					}
				}
				
			}
			
			this.showBreadcrumbs();
		},
		
		_getProcessAreaStore: function(){
			if (!this.sharedStore) {
				var args = {
					editorContext: ProcessConstants.EditorContext.PROJECT,
					processAreaItemId: this.projectAreaItemId,
					errorContext: this,
					errorCallback: this._handleCreateStoreError
				};
				this.sharedStore = new com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore(args);
			}
			return this.sharedStore;
		},
		
		// called from AbstractProcessAreaEditor.postCreate
		customConnect: function(){
		
			if (!this.isNew) {
				this.connect(this.createChildButton, ON_CLICK, function(e){ //$NON-NLS-1$
					dojo.stopEvent(e);
					var name = this.name.savedValue;
					if (!this._handleNavigate()) {
						document.location.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=new&parentItemId=" + this.projectAreaItemId + "&projectAreaItemId=" + this.projectAreaItemId + "&parentName=" + encodeURIComponent(name) + "&parentType=PROJECT"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
					}
				});
				this.teamAreaHierarchy.style.display = ProcessConstants.EMPTY;
			}
			
			this.connect(this.linkAddButton, ON_CLICK, function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				this.addProjectLink();
			});
			
		},
		
		addProjectLink: function(){
			var addLink = new com.ibm.team.process.web.ui.internal.admin.projects.AddProjectLinkDialog({
				projectAreaItemId: this.projectAreaItemId,
				projectAreaName: this.name.value,
				originalProjectAreaName: this.name.savedValue
			});
			var self = this;
			var dialogWidget = new jazz.ui.Dialog({
				primaryTitle: this.messages.ProjectAreaEditor_AddProjectLinkDialogTitle,
				contentNode: addLink.domNode,
				width: dojo.isIE ? "30em" : "32em", //$NON-NLS-1$ //$NON-NLS-2$ 
				destroyContents: true
			});
			dialogWidget.addLinkObj = addLink;
			addLink.startup();
			//	addLink.serviceProvidersList.focus();
			this.connectHandles.push(dojo.connect(addLink.cancelButton, ON_CLICK, dialogWidget, function(e){
				dojo.stopEvent(e);
				this.close();
			}));
			this.connectHandles.push(dojo.connect(addLink.finishButton, ON_CLICK, dialogWidget, function(e){
				self._addNewProjectLink(this.addLinkObj.getAssociatedProject());
				this.close();
			}));
			this.connectHandles.push(dojo.connect(addLink.crossServerLink, ON_CLICK, dialogWidget, function(e){
				this.close();
			}));
			
		},
		
		_addNewProjectLink: function(newProjectAssociation){
			//disable all other rows to highlight the new one
			var rows = this.linkArea.rows;
			for (var i = 0; i < rows.length; i++) {
				dojo.toggleClass(rows[i], ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
			}
			var existingLink;
			for (var j = 0; j < this.existingProjectAreaLinks.length; j++) {
				var existingProjectAreaLink = this.existingProjectAreaLinks[j];
				if (newProjectAssociation.linkType == existingProjectAreaLink.linkType && newProjectAssociation.projectAreaInfo.targetUrl == existingProjectAreaLink.projectAreaInfo.targetUrl && newProjectAssociation.projectAreaInfo.servicesUrl == existingProjectAreaLink.projectAreaInfo.servicesUrl) {
					//attempting to add a link that already exists or was deleted and re-added between saves
					existingLink = existingProjectAreaLink;
					break;
				}
			}
			
			if (existingLink) { //removed and re-added between saves
				for (j = 0; j < this.removedProjectAreaLinks.length; j++) {
					if (existingLink === this.removedProjectAreaLinks[j]) {
						this.removedProjectAreaLinks.splice(j, 1);
						this.projectAreaLinks.push(existingLink);
						this._createProjectLinkRow(existingLink, true);
						this.setDirty(this.changesPending());
						return;
					}
				}
			} else {
				this.projectAreaLinks.push(newProjectAssociation);
				if (!this.newProjectAreaLinks) {
					this.newProjectAreaLinks = [];
				}
				this.newProjectAreaLinks.push(newProjectAssociation);
				this._createProjectLinkRow(newProjectAssociation, true);
				this.setDirty(this.changesPending());
				return;
			}
			var tr = this._getRowForLink(existingLink);
			dojo.toggleClass(tr, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
		},
		
		_getRowForLink: function(existingLink){
			var rows = this.linkArea.rows;
			for (var i = 0; i < rows.length; i++) {
				var rowProjectLink = rows[i].projectAreaLink;
				if (existingLink == rowProjectLink) {
					return rows[i];
				}
			}
			return null;
		},
		
		populateProjectAreaLinks: function(){
			dojox.data.dom.removeChildren(this.linkArea);
			if (this.projectAreaLinks) {
				for (var i = 0; i < this.projectAreaLinks.length; i++) {
					this._createProjectLinkRow(this.projectAreaLinks[i], false);
				}
			}
		},
		
		_createProjectLinkRow: function(projectAreaLink, select){
			var tr = document.createElement("tr"); //$NON-NLS-1$
			var projectAreaInfo = projectAreaLink.projectAreaInfo;
			tr.projectAreaLink = projectAreaLink;
			
			var td = document.createElement("td"); //$NON-NLS-1$
			td.className = "linkLabel"; //$NON-NLS-1$
			td.appendChild(document.createTextNode(projectAreaLink.linkLabel));
			tr.appendChild(td);
			
			td = document.createElement("td"); //$NON-NLS-1$
			td.className = "linkServiceProvider"; //$NON-NLS-1$
			var link = document.createElement("a"); //$NON-NLS-1$
			link.href = projectAreaInfo.presentationUrl;
			link.appendChild(document.createTextNode(decodeURIComponent(projectAreaInfo.label)));
			td.appendChild(link);
			tr.appendChild(td);
			
			td = document.createElement("td"); //$NON-NLS-1$
			td.className = "iconNode"; //$NON-NLS-1$
			var trToolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();
			td.appendChild(trToolbar.domNode);
			this._actionToolbars.push(trToolbar);
			this.addRemoveIcon(trToolbar, this._removeProjectAreaLink);
			tr.appendChild(td);
			
			this.connectHandles.push(dojo.connect(tr, "onmouseover", tr, function(e){ //$NON-NLS-1$
				dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
			}));
			this.connectHandles.push(dojo.connect(tr, "onmouseout", function(e){ //$NON-NLS-1$
				dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
			}));
			this.connectHandles.push(dojo.connect(link, "onfocus", tr, function(e){ //$NON-NLS-1$
				dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
			}));
			this.connectHandles.push(dojo.connect(link, "onblur", tr, function(e){ //$NON-NLS-1$
				dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
			}));
			
			this.linkArea.appendChild(tr);
			if (select) {
				dojo.toggleClass(tr, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
			}
		},
		
		_removeProjectAreaLink: function(e){
			dojo.stopEvent(e);
			var rowNode = e.target;
			while (rowNode.tagName !== "TR") { //$NON-NLS-1$
				rowNode = rowNode.parentNode;
			}
			var selectedLink = rowNode.projectAreaLink;
			
			if (!selectedLink.linkUrl) {
				for (var j = 0; j < this.newProjectAreaLinks.length; j++) {
					if (selectedLink === this.newProjectAreaLinks[j]) {
						this.newProjectAreaLinks.splice(j, 1);
						break;
					}
				}
			} else { //not a link that was just added
				this.removedProjectAreaLinks.push(selectedLink);
			}
			
			for (j = 0; j < this.projectAreaLinks.length; j++) {
				if (this.projectAreaLinks[j] === selectedLink) {
					this.projectAreaLinks.splice(j, 1);
					break;
				}
			}
			this.setDirty(this.changesPending());
			rowNode.parentNode.removeChild(rowNode);
		},
		
		_createTableHeader: function(){
			if (this.linkHeader.childNodes.length > 1) {
				return; //already created
			}
			var tr = document.createElement("tr"); //$NON-NLS-1$
			tr.className = "headerLineRow"; //$NON-NLS-1$
			var th = document.createElement("th"); //$NON-NLS-1$
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" ")); //$NON-NLS-1$
			}
			th.className = "linkLabel"; //$NON-NLS-1$
			tr.appendChild(th);
			th = document.createElement("th"); //$NON-NLS-1$
			th.className = "linkServiceProvider"; //$NON-NLS-1$
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" ")); //$NON-NLS-1$
			}
			tr.appendChild(th);
			th = document.createElement("th"); //$NON-NLS-1$
			th.className = "iconNode"; //$NON-NLS-1$
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" ")); //$NON-NLS-1$
			}
			tr.appendChild(th);
			
			this.linkHeader.appendChild(tr);
			
			tr = document.createElement("tr"); //$NON-NLS-1$
			th = document.createElement("th"); //$NON-NLS-1$
			th.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_ProjectAreaLinkHeader2));
			th.className = "linkLabel"; //$NON-NLS-1$
			tr.appendChild(th);
			
			th = document.createElement("th"); //$NON-NLS-1$
			th.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_ProjectAreaLinkHeader));
			th.className = "linkServiceProvider"; //$NON-NLS-1$
			tr.appendChild(th);
			
			th = document.createElement("th"); //$NON-NLS-1$
			th.className = "iconNode"; //$NON-NLS-1$
			dojo.addClass(th, ProcessConstants.CSSClass.ACTION_COLUMN);
			dojo.style(th, "width", "15%"); //$NON-NLS-1$ //$NON-NLS-2$
			th.appendChild(document.createTextNode(this.messages.ProjectAreaEditor_ProjectAreaLinkActionsHeader));
			tr.appendChild(th);
			
			this.linkHeader.appendChild(tr);
		},
		
		_getProjectAreaLinksURL: function(){
			return this._getThisTargetUrl() + "/links"; //$NON-NLS-1$
		},
		
		_fetchProjectAreaLinks: function(){
			var projectAreasLinksUrl = this._getProjectAreaLinksURL();
			jazz.client.xhrGet({
				url: projectAreasLinksUrl,
				headers: {
					"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
				},
				handleAs: "xml", //$NON-NLS-1$
				load: dojo.hitch(this, this._createProjectAreaLinks),
				error: dojo.hitch(this, this._handleErrorGettingProjectAreaLinks)
			});
		},
		
		_createProjectAreaLinks: function(xmlDocument){
			if (!xmlDocument) {
				return;
			}
			
			var children = xmlDocument.childNodes;
			var linksNode = UIHelper.getNode(children, "links", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
			var linkChildren = linksNode.childNodes; //children of ServiceProviderCatalog
			var projectAreas = UIHelper.getNodes(linkChildren, "link", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
			var count = projectAreas.length;
			dojo.forEach(projectAreas, function(projectAreaLink){
				var url = projectAreaLink.getAttribute("projectArea");//$NON-NLS-1$
				var presentationUrl = url;
				var projectAreaLinkChildren = projectAreaLink.childNodes;
				
				var urlNode = UIHelper.getNode(projectAreaLinkChildren, "url", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
				var linkUrl = urlNode.firstChild.data;
				
				var linkTypeNode = UIHelper.getNode(projectAreaLinkChildren, "link-type", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
				var linkType = linkTypeNode.firstChild.data;
				
				var serviceProviderNode = UIHelper.getNode(projectAreaLinkChildren, "ServiceProvider", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				var serviceProviderChildren = serviceProviderNode.childNodes;
				
				var titleNode = UIHelper.getNode(serviceProviderChildren, "title", ProcessConstants.NAMESPACE_DC); //$NON-NLS-1$
				var label = titleNode.firstChild.data;
				
				var detailsNode = UIHelper.getNode(serviceProviderChildren, "details", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				var attributeNode = UIHelper.getAttributeNode(detailsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				var targetUrl = attributeNode.value;
				
				var servicesURLNode = UIHelper.getNode(serviceProviderChildren, "services", ProcessConstants.NAMESPACE_OSLC); //$NON-NLS-1$
				attributeNode = UIHelper.getAttributeNode(servicesURLNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				var servicesUrl = attributeNode.value;
				
				var consumerRegistryNode = UIHelper.getNode(serviceProviderChildren, "consumerRegistry", ProcessConstants.NAMESPACE_JP); //$NON-NLS-1$
				attributeNode = UIHelper.getAttributeNode(consumerRegistryNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
				var consumerRegistryUrl;
				if (attributeNode) { //a consumer registry does not have to be provided to have a valid link
					consumerRegistryUrl = attributeNode.value;
				}
				
				var associatedProjectLink = {};
				var associatedProjectAreaInfo = {};
				associatedProjectLink.projectAreaInfo = associatedProjectAreaInfo;
				associatedProjectAreaInfo.label = label;
				associatedProjectAreaInfo.targetUrl = targetUrl;
				associatedProjectAreaInfo.servicesUrl = servicesUrl;
				associatedProjectAreaInfo.presentationUrl = targetUrl;
				associatedProjectAreaInfo.consumerRegistryUrl = consumerRegistryUrl;
				associatedProjectLink.linkLabel = this.getLinkLabel(linkType);
				associatedProjectLink.linkType = linkType;
				associatedProjectLink.linkUrl = linkUrl;
				
				this.projectAreaLinks.push(associatedProjectLink);
				this.existingProjectAreaLinks.push(associatedProjectLink);
				
				count = count - 1;
				if (count === 0) {
					this.populateProjectAreaLinks();
				}
				
			}, this);
		},
		
		_handleErrorGettingProjectAreaForLink: function(error){
			if (error.responseText) {
				var errorHeader = dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorLoadingProjectAreaForLinkWithReason, [error.responseText]);
			} else {
				errorHeader = this.messages.ProjectAreaEditor_ErrorLoadingProjectAreaForLink;
			}
			this.handleServerResponseError(error, errorHeader);
		},
		
		_handleErrorGettingProjectAreaLinks: function(error){
			if (error.responseText) {
				var errorHeader = dojo.string.substitute(this.messages.ProjectAreaEditor_ErrorLoadingLinksWithReason, [error.responseText]);
			} else {
				errorHeader = this.messages.ProjectAreaEditor_ErrorLoadingLinks;
			}
			this.handleServerResponseError(error, errorHeader);
		},
		
		customChangesPending: function(){
			return this.linksHaveChanged();
		},
		
		linksHaveChanged: function(){
			return this.newProjectAreaLinks.length > 0 || this.removedProjectAreaLinks.length > 0;
		},
		
		getDeferredSaveCount: function(response){
			if (!this.isNew || response) {
				//return the total number of deferred calls that will occur
				var count = 1; //repository save of the project area and link creation
				if (response) {
					//the repository save has already occurred for the new area
					count = 0;
				}
				
				dojo.forEach(this.newProjectAreaLinks, function(projectAreaLink){
					//new back links POST
					var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
					if (backLinkConsumerRegistryUrl) {
						count++;
					}
				}, this);
				
				dojo.forEach(this.removedProjectAreaLinks, function(projectAreaLink){
					//remove back links DELETE
					var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
					if (backLinkConsumerRegistryUrl) {
						count++;
					}
				}, this);
				
				if (this._tabs) {
					for (tabId in this._tabs) {
						var tab = this._tabs[tabId];
						if (this.isActivatedExtensionTab(tab)) {
							if (tab.mainPage.changesPending()) {
								count++;
							}
						}
					}
				}
				
				return count;
			} else {//for new areas we do the repo save and on success post the links
				return this.inherited(arguments);
			}
		},
		
		handleSaveResponse: function(response){
			if (this.isNew && response && this.newProjectAreaLinks.length > 0) {
				this.deferredSaveCount = this.getDeferredSaveCount(response);
				//do not have to deal with removed as since it is a new project
				//area there cannot be any removed links
				this.projectAreaItemId = response;
				var createTrace = this.messages.ProjectAreaEditor_CreatingBackLinks;
				dojo.forEach(this.newProjectAreaLinks, function(projectAreaLink){
					var backLinkConsumerRegistryUrl = this._getBackLinkConsumerRegistryUrl(projectAreaLink);
					if (backLinkConsumerRegistryUrl) {
						createTrace = "\n\t" + dojo.string.substitute(this.messages.ProjectAreaEditor_CreatingBackLinkOfType, [projectAreaLink.linkType, projectAreaLink.projectAreaInfo.label]); //$NON-NLS-1$
						this.postBackLink(projectAreaLink, createTrace);
					}
				}, this);
				this.newProjectAreaLinks = [];
			}
			
			this._saveExtensionTabs();
			
			this.inherited(arguments);
		},
		
		_saveExtensionTabs: function(){
			if (!this.isNew && this._tabs) {
				for (tabId in this._tabs) {
					var tab = this._tabs[tabId];
					if (this.isActivatedExtensionTab(tab)) {
						if (tab.mainPage.changesPending()) {
							tab.mainPage.doSave(dojo.hitch(this, this._saveExtensionPageCallback));
						}
					}
				}
			}
		},
		
		
		/*
		 * @param Error errorObject
		 *
		 * @return void
		 */
		_saveExtensionPageCallback: function(errorObject){
		
			if (!this.domNode) {
				// Edge case
				// Notification too late, the editor has already been destroyed
				return;
			}
			
			if (errorObject) {
				// Save operation failed for the tab
				this.handleServerResponseError(errorObject);
				this.saveButton.disabled = false;
			} else {
				// Save operation passed for the tab
				this.handleServerResponseSuccess();
			}
		},
		
		getLinkLabel: function(linkType){
			switch (linkType) {
				case ProcessConstants.LinkTypes.IMPLEMENTS:
					return this.messages.LinkType_Implements;
				case ProcessConstants.LinkTypes.IMPLEMENTED_BY:
					return this.messages.LinkType_Implemented;
				case ProcessConstants.LinkTypes.VALIDATES:
					return this.messages.LinkType_Validates;
				case ProcessConstants.LinkTypes.VALIDATED_BY:
					return this.messages.LinkType_Validated;
				case ProcessConstants.LinkTypes.TESTS:
					return this.messages.LinkType_Tests;
				case ProcessConstants.LinkTypes.TESTED_BY:
					return this.messages.LinkType_Tested;
				case ProcessConstants.LinkTypes.RELATED_TO:
					return this.messages.LinkType_RelatedTo;
			}
			return linkType;
		},
		
		uninitialize: function(){
			if (this.accessControlEditor) {
				this.accessControlEditor.destroyRecursive();
				delete this.accessControlEditor;
			}
			if (this.timelineEditor) {
				this.timelineEditor.destroyRecursive();
				delete this.timelineEditor;
			}
			this.inherited(arguments);
		}
		
	});
})();
