/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TimelineTree"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var TimlineTreeCSSClass = {
		NEW_PROCESS_ITEM: 'newProcessItem' //$NON-NLS-1$
	};
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TimelineTree", [com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree], { //$NON-NLS-1$
		/*
		 * Constructs the Timeline tree instance
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 *    model: the model the tree should use for it's data
		 *    editorContext: the editor context tree is being used in... a value in ProcessConstants.EditorContext
		 *    onClick: the function to call when a node in the tree is clicked on
		 *  }
		 */
		constructor: function(args){
			// Place holder for now
		},
		
		getLabel: function(item){
			var label = this._getDisplayLabel(item);
			if (item.enabledContext == ProcessConstants.PermissionContext.DEV_LINE || item.enabledContext == ProcessConstants.PermissionContext.ITERATION) {
				var dateDecorator = this._getDateDecorator(item);
				if (item.projectTimeline == "true") { //$NON-NLS-1$
					label = dojo.string.substitute(this.messages.TimelineTree_TimelineDecoratorText, [label, this.messages.TimelineTree_ProjectTimelineDecoratorText, dateDecorator]);
				} else {
					label = dojo.string.substitute(this.messages.TimelineTree_IterationDecoratorText, [label, dateDecorator]);
				}
			}
			return label;
		},
		
		getLabelClass: function(item, opened){
			if (item && !item.root) {
				var store = this.model.store;
				if (store.isTimeline(item) || store.isIteration(item)) { //$NON-NLS-1$
					if (this.isNewItem(item)) {
						return TimlineTreeCSSClass.NEW_PROCESS_ITEM;
					}
				}
				
				return this.inherited(arguments);
				
			}
		},
		
		addNewTimeline: function(timeline){
			if (this.lastFocused) {
				dojo.removeClass(this.lastFocused.labelNode, "dijitTreeLabelFocused"); //$NON-NLS-1$
			}
			var newTimeline = this.model.store.addNewTimeline(timeline);
			return newTimeline;
		},
		
		addNewIteration: function(iteration){
			if (this.lastFocused) {
				dojo.removeClass(this.lastFocused.labelNode, "dijitTreeLabelFocused"); //$NON-NLS-1$
				if (!this.lastFocused.isExpanded) {
					this.lastFocused.expand();
				}
			}
			var parentItem = this.getFocusedItem();
			var parentId = this.model.getIdentity(parentItem);
			iteration.parentId = parentId;
			var parentContextId = this.model.store.getValue(parentItem, "contextId"); //$NON-NLS-1$
			iteration.id = parentContextId + ProcessConstants.PERMISSION_PATH_SEPARATOR + iteration.id;
			//  Fatty doesn't change the state of the icon until after save...should we?
			//  Commenting out until we decide
//			iteration.current = this.model.store.getValue(parentItem, "current") ? true : false; //$NON-NLS-1$
//			iteration.completed = this.model.store.getValue(parentItem, "completed") ? true : false; //$NON-NLS-1$
//			if (this.model.store.getValue(parentItem, "leaf")) { //$NON-NLS-1$
//				iteration.leaf = true;
//				this.model.store.setValue(parentItem, "leaf", false); //$NON-NLS-1$
//			} else {
//				iteration.leaf = false;
//			}
			var newIteration = this.model.store.addNewIteration(iteration);
			return newIteration;
		},
		
		updateIteration: function(iteration, iterationData) {
			var store = this.model.store;
			//  No Edit support for the id right now so don't update it unless the iteration is new. 
			//  Especially since we stripped off the entire context path for existing
			if (!this.isNewItem(iteration)) {
				iterationData.contextId = store.getValue(iteration, "contextId"); //$NON-NLS-1$
			}
			store.updateIteration(iteration, iterationData);
		},

		deleteSelectedItem: function(){
			if (this.lastFocused) {
				this.model.store.deleteItem(this.lastFocused.item);
			}
		},
		
		getFocusedItem: function(){
			if (this.lastFocused) {
				return this.lastFocused.item;
			}
			return null;
		},
		
		isNewItem: function(item){
			var id = this.model.getIdentity(item);
			if (id.indexOf(ProcessConstants.EditorMode.NEW) === 0) {
				return true;
			}
			return false;
		},
		
		_getDateDecorator: function(item){
			var dateDecorator = ""; //$NON-NLS-1$
			var store = this.model.store;
			var startDate;
			var endDate;
			if (store.hasAttribute(item, "startDate")) { //$NON-NLS-1$	
				startDate = this._formatDate(store.getValue(item, "startDate")); //$NON-NLS-1$
				if (store.hasAttribute(item, "endDate")) { //$NON-NLS-1$	
					endDate = this._formatDate(store.getValue(item, "endDate")); //$NON-NLS-1$
				}
			}
			if (startDate && startDate !== this.messages.TimelineTree_DateUnknown) {
				dateDecorator = dojo.string.substitute(this.messages.TimelineTree_DateDecorator, [startDate, endDate]);
			}
			return dateDecorator;
		},
		
		_formatDate: function(date){
			if (date) {
				return dojo.date.locale.format(date, {
					selector: "date", //$NON-NLS-1$
					formatLength: "short" //$NON-NLS-1$
				});
			} else {
				return this.messages.TimelineTree_DateUnknown;
			}
		},
		
		_focusChanged: function(node){
		
		},
		
		uninitialize: function(){
			// Place holder for now
			this.inherited(arguments);
		},
		
		/*
		 * The methods below this point were taken from Tree.js so that we could
		 * intercept clicks on the icon and still allow the keyboard to function correctly
		 * in the tree.  As Tree.js changes we'll need to update these appropriately.
		 */
		focusNode: function(/* _tree.Node */node){ // from Tree.js
			// set focus so that the label will be voiced using screen readers
			node.labelNode.focus();
			
			// Process related additions
			this._focusChanged(node);
			
		},
		
		_onBlur: function(){
			if (this.lastFocused) {
				var labelNode = this.lastFocused.labelNode;
				// Process related additions
				// we want to see what was last focused in the tree
				// dojo.removeClass(labelNode, "dijitTreeLabelFocused");	
			}
		},
		
		_onTreeFocus: function(/*Widget*/node){
			// summary:
			//		called from onFocus handler of treeitem labelNode to set styles, wai state and tabindex
			//		for currently focused treeitem.
			
			if (node) {
				if (node != this.lastFocused) {
					this.blurNode();
				}
				var labelNode = node.labelNode;
				// set tabIndex so that the tab key can find this node
				labelNode.setAttribute("tabIndex", "0");   //$NON-NLS-1$ //$NON-NLS-2$
				dijit.setWaiState(labelNode, "selected", true); //$NON-NLS-1$
				dojo.addClass(labelNode, "dijitTreeLabelFocused"); //$NON-NLS-1$
				this.lastFocused = node;
				
				// process related addition
				// tree doesn't focus the node when tabbing in to the tree
				this.focusNode(node);
			}
		}
	
	
	
	});
})();
