﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
class LayoutManager {

	private var _debugArea:DebugArea;
	
	private var _legend:Legend;
	
	private var _chart:IChart;
	
	// Legend can only occupy up to a maxiumum of 38% of available chart area width
	private static var _MAX_LEGEND_AREA_WIDTH_RATIO:Number = 0.38;  
	
	
	public function LayoutManager(chartArea:ChartArea) {
		this._legend = chartArea.getLegend();
		this._chart = chartArea.getChart();
		this._debugArea = chartArea.getDebugArea();
	}

	public function layout(parentMC:MovieClip):Void {
		
		this._configureChartObjects();
		this._drawChartObjects(parentMC);
		
	}
	
	private function _configureChartObjects():Void {
		// Debug Area
		if (this._debugArea) {
			this._debugArea.setWidth(this._debugArea.getPreferredWidth());
			this._debugArea.setHeight(this._debugArea.getPreferredHeight());
			this._debugArea.setStartX(0);
			this._debugArea.setStartY(0);
		}

		// TODO: Determine where best to calculate chart margins
		// Legend
		if (this._legend) {
			this._legend.setWidth(this._calculateLegendWidth());
			this._legend.setHeight(this._calculateLegendHeight());
		}
		
		// Chart
		this._chart.setWidth(this._calculateChartWidth());
		this._chart.setHeight(this._calculateChartHeight());
		this._chart.setStartX(this._calculateChartStartX());
		this._chart.setStartY(this._calculateChartStartY());		
		
		// Legend
		if (this._legend) {
			this._legend.setStartX(this._calculateLegendStartX());
			this._legend.setStartY(this._calculateLegendStartY());			
		}		
		
	}
	
	private function _drawChartObjects(parentMC:MovieClip):Void {
		// Debug Area
		if (this._debugArea) {
			this._debugArea.draw(parentMC);
		}
		
		// Chart
		this._chart.draw(parentMC);
		
		// Legend
		if (this._legend) {
			this._legend.draw(parentMC, this._chart.getShapes());
		}		
	}
	
	private function _calculateLegendWidth():Number {
		
		var preferredWidth:Number = this._legend.getPreferredWidth();
		var maxWidth:Number = Math.floor(Stage.width * LayoutManager._MAX_LEGEND_AREA_WIDTH_RATIO);
		
		if (preferredWidth > maxWidth) {
			return maxWidth;
		}
		return preferredWidth;
	}	

	private function _calculateLegendHeight():Number {
		
		var targetHeight:Number;
		var preferredHeight:Number = this._legend.getPreferredHeight();
		var maxHeight:Number = Stage.height -
								ChartArea.MARGIN -
								ChartArea.MARGIN;

		if (this._debugArea) {
			maxHeight = maxHeight -
						this._debugArea.getHeight();
		}
		
		if (preferredHeight > maxHeight) {
			targetHeight = maxHeight;
		} else {
			targetHeight = preferredHeight;
		}

		if (targetHeight < 0) {
			targetHeight = 0;
		}
		
		return targetHeight;
	}	

	private function _calculateLegendStartX():Number {
		return this._chart.getStartX() + this._chart.getWidth();
	}	
	
	private function _calculateLegendStartY():Number {
		
		var midpoint:Number;

		if (this._debugArea) {
			midpoint = (Stage.height / 2) + (this._debugArea.getHeight() * 0.5);
		} else {
			midpoint = Stage.height / 2;
		}
		var startY:Number = midpoint - (this._legend.getHeight() * 0.5);

		if (startY < ChartArea.MARGIN) {
			return ChartArea.MARGIN;
		}

		return startY;
	}	
	
	private function _calculateChartWidth():Number {
		if (this._legend) {
			return this._chart.getPreferredWidth() - this._legend.getWidth();
		}		
		return this._chart.getPreferredWidth();
	}

	private function _calculateChartHeight():Number {
		if (this._debugArea) {
			return this._chart.getPreferredHeight() - this._debugArea.getHeight();
		}
		return this._chart.getPreferredHeight();
	}
	
	private function _calculateChartStartX():Number {
		return ChartArea.MARGIN;
	}	

	private function _calculateChartStartY():Number {
		if (this._debugArea) {
			return ChartArea.MARGIN + this._debugArea.getHeight();
		}
		return ChartArea.MARGIN;
	}	
	
	public function destroy():Void {
		// TODO: Make into interface method?
		// Note: Chart area already destroys these
		// Debug area
		if (this._debugArea) {
			this._debugArea.destroy();
			delete this._debugArea;
		}
		// Legend
		if (this._legend) {
			this._legend.destroy();
			delete this._legend;
		}
		// Chart
		if (this._chart) {
			this._chart.destroy();
			delete this._chart;
		}
	}		
	
}