/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.module.AbstractChooserWidget");

// declare dependencies on other Dojo modules
dojo.require("jazz.ui.RemoteChooserResponder");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "ModuleMessages");

(function() {

var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;

var PROCESS_SERVICE_URI = "com.ibm.team.process.internal.common.service.IProcessRestService";
var CONTRIBUTOR_SERVICE_URI = "com.ibm.team.repository.common.internal.IContributorRestService";

var ITEM_KEY_MESSAGE= "oslc_cm:message";

var ITEM_KEY_RESULT= "oslc_cm:results";
var ITEM_KEY_URI= "rdf:resource";
var ITEM_KEY_LINKTITLE= "oslc_cm:label";

// declare and implement the widget "class"
dojo.declare("com.ibm.team.workitem.web.ui.internal.module.AbstractChooserWidget", [ dijit._Widget, dijit._Templated ], {

templatePath: 	dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/module/templates/AbstractChooserWidget.html"),

constructor: function(params) {
	var searchString= location.search.replace(/^\?/, ''); // remove leading '?'
	this._dialogArguments= dojo.queryToObject(searchString);
	
	this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "ModuleMessages");
	
},

postCreate: function() {
    // override the styles from the UI module (minWith big, blue background)
    var body= dojo.body();
    dojo.style(body, "minWidth", "200px");
    dojo.style(body, "background", "none");
    
    dojox.data.dom.replaceChildren(this._loadingMessage, document.createTextNode(this.messages.busyLoading));
    
	this._findAuthenticatedContributor();
},

_findAuthenticatedContributor: function() {
	var self= this;
	var handlerObj= {
		success: function(contributor) {
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", contributor); //$NON-NLS-1$
		
			self._evaluateProjectAreas(contributor);
		},
		failure: function(errorObj) {
			self._handleError(self.messages.msgNoContributorFound);
		}
	};

  	var srh = new ServiceResponseHandler(handlerObj, "success", "failure");
   	var serviceRequest = new ServiceRequest(CONTRIBUTOR_SERVICE_URI, "getCurrentContributor", null);
   	TeamServerClient.invokeService(serviceRequest, srh);
}, 
_handleError: function(msg) {
	alert(msg);
}, 
_evaluateProjectAreas: function(contributor) {
	var self= this;
	var handlerObj= {
		success: function(projectAreas) {
		    var name= self._dialogArguments.projectAreaName;
		    var itemId= self._dialogArguments.projectAreaId;
		    
		    for (var i = 0; i < projectAreas.length; i++) {
		        var pa= projectAreas[i];
		        if ((name && (name == pa.name)) || (itemId &&  (itemId == pa.itemId))) {	        
		        	dojo.setObject("com.ibm.team.process.web.projectArea", pa);
		            self._openDialog(pa);
		            return;
		        }
		    }
		    var msg = dojo.string.substitute(self.messages.msgNoProjectAreaFound, [name || itemId ]);
		    self._handleError(msg);
		},
		failure: function(errorObj) {
			self._handleError(self.messages.errorCannotFetchProjectArea);
		}
	};

   var srh = new ServiceResponseHandler(handlerObj, "success", "failure");
   var authContribId = contributor.userId;
   var serviceRequest = new ServiceRequest(PROCESS_SERVICE_URI, "getAllProjectAreas", {hideArchivedProjects: true, userId: authContribId});
   TeamServerClient.invokeService(serviceRequest, srh);
},

_openDialog: function(projectArea) {    
   throw new Exception("_openDialog must be implemented by sub classes");
},

_handleError: function(errorMessage) {
    alert(errorMessage);
    this._close(null, null);
},
    
_setContentWidget: function(domNode) {
    this._itemPickerNode.removeChild(this._loadingMessage);
    this._itemPickerNode.appendChild(domNode);
},

_close: function(resultArray, returnMessage) {
   var newItems= [];

   if (resultArray) { // expected [ { id, summary } ]
	    for (var i=0; i<resultArray.length; i++) {
	        var item= resultArray[i];
	        
	        var name= item.id + ": " + item.summary;
	        var itemUrl=  location.protocol + '//' + location.host + net.jazz.ajax._contextRoot + "/resource/itemName/com.ibm.team.workitem.WorkItem/" +  item.id;
	        var newItem= { "rdf:resource": itemUrl, "oslc_cm:label" : name };
	        
	        newItems.push(newItem);
	    }
	}

    var response= {
      "oslc_cm:message" : returnMessage,
      "oslc_cm:results" : newItems
    };
    var remoteFrame= new jazz.ui.RemoteChooserResponder();
    remoteFrame.sendResponse(dojo.toJson(response));
}
	
});

})();
