/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.users.AbsencesLabelProvider"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.ImageClient") //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.viewers.ActionColumnLabelProvider"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.users.dialogs.AbsenceEditDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
	
var ImageClient				= com.ibm.team.apt.web.client.internal.ImageClient;
var AbsenceEditDialog		= com.ibm.team.apt.web.ui.internal.users.dialogs.AbsenceEditDialog;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	
dojo.declare("com.ibm.team.apt.web.ui.internal.users.AbsencesLabelProvider", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.viewers.ActionColumnLabelProvider], {
	
	constructor: function() {
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getLabel: function(element, index) {
		if (index == 0) {
			return element.description;
		} else if (index == 1) {
			var startDate= element.startDate;
			var endDate= element.endDate;
			
			var startDateText= dojo.date.locale.format(startDate, { selector: "date", formatLength: "medium" }); //$NON-NLS-1$ //$NON-NLS-2$
			var endDateText= dojo.date.locale.format(endDate, { selector: "date", formatLength: "medium" }); //$NON-NLS-1$ //$NON-NLS-2$
			
			if (startDateText == endDateText) {
				return startDateText;
			} else {
				return bind(MSGS.USER_EDITOR_ABSENCES_DATE, [startDateText, endDateText]);
			}
		}
	},
	
	getIcon: function(element, index) {
		if (index == 0) {
			var icon= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/users/images/outofoffice_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
			return ImageClient.getImageUrl(icon);
		}
	},
	
	destroy: function() {		
		this.inherited(arguments);
	},
	
	// ---- SPI -------------------------------------------------
	
	_getActionColumnIndex: function() {
		return 2;
	},
	
	_createActions: function(element) {
		var result= new Array();
		result.push(this.__createDeleteAction(element));
		result.push(this.__createEditAction(element));
		return result;
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__createDeleteAction: function(element) {
		return this._createAction(
				MSGS.USER_EDITOR_ABSENCES_ACTIONS_DELETE_LABEL, 
				ImageClient.getImageUrl(dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/users/images/delete.gif")), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.hitch(this, function() {
					this.__doDelete(element);
				})
		);
	},
	
	__createEditAction: function(element) {
		return this._createAction(
				MSGS.USER_EDITOR_ABSENCES_ACTIONS_EDIT_LABEL, 
				ImageClient.getImageUrl(dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/users/images/edit.gif")), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.hitch(this, function() {
					this.__doEdit(element);
				})
		);
	},
		
	__doDelete: function(element) {
		element.getModel().removeAbsence(element);
	},
	
	__doEdit: function(element) {
		var _self= this;
		new AbsenceEditDialog({
			absence: element,
			onOk: function() {
				element.getModel().markModified(element);
			},
			onCancel: function() {
			}
		});
	},

	__sentinel: null // terminates this class definition
});

})();
