/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.utils.ParameterNameChooserDialog");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.ListBox");

dojo.requireLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "StatisticsMessages");

/*
 * Aliases 
 */
var Dialog = jazz.ui.Dialog;
var ListBox = jazz.ui.ListBox;


(function() {

/*
 * Class 
 */
dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.utils.ParameterNameChooserDialog",
	[dijit._Widget, dijit._Templated], {
	
	/*
	 * Properties
	 */		
	parameterNames: [],
	parameterValues: [],
	queryItemID: null,
	parameterID: null,
	onOK: null,
	onCancel: null,	
	iconProvider: null,
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "ui/internal/templates/utils/ParameterNameChooserDialog.html"),

	/*
 	 * @param Object args
 	 * 
 	 * @return void
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "StatisticsMessages");
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		this.inherited(arguments);
		
		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->

		this._dialog = null;
		this._listBox = null;
		
		this._selectedParameterName = null;
		this._selectedParameterValue = null;
		this._doIt = false;
		
		// Initialization //--------------------------------------------------->

		// Create the UI
		this._createDialogContents();
		this._connectDialogHandlers();
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		this._disconnectDialogHandlers();
		
		// List box
		if (this._listBox) {
			this._listBox.destroy();
			delete this._listBox;
		}
		// Dialog
		if (this._dialog) {
			this._dialog.destroy();
			delete this._dialog;
		}

		this.inherited(arguments);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	show: function() {
		// Create the dialog
		this._dialog = new Dialog({
			contentNode: this.domNode,
			primaryTitle: this.messages.msgSelectParameterName,
			width: "330px",
			onClose: dojo.hitch(this, this._handleEventDialogClosed) 
		});		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	hide: function() {
		if (this._dialog) {
			this._dialog.close();
		}		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventDialogClosed: function() {

		if (this._doIt && 
			this.onOK) {
			// OK button clicked
			this.onOK(this);
		} else if (this.onCancel) {
			// Cancel button clicked
			this.onCancel(this);
		}
		
		var self = this;
		setTimeout(function() {
			self.destroy();
		}, 1000);
		
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */	
	getSelection: function() {
		var selection = {
			name: this._selectedParameterName,
			value: this._selectedParameterValue
		};
		return selection;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_createDialogContents: function() {
		this._createUIListBox();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_createUIListBox: function() {
		
		// Create the list box
		this._listBox = new ListBox({
			allowMultipleSelections: false,
			height: this._getListBoxHeight(),
			width: "300px",
			margin: "10px",
			sortField: "type",
			iconMap: this._getIconMap(),
			ondblClick: dojo.hitch(this, this._handleEventEntryDoubleClicked)
		});		
		
		// Populate the list box
		for (var index = 0; index < this.parameterNames.length; index++) {
			
			var label = this.parameterNames[index];
			var value = this.parameterValues[index];
			var labelHTML = "<span title=\"" + value + "\">" + label + "</span>" +
							"<span style=\"color:#AAAAAA\"> (" + value + ")</span>";
			
			var entry = {
				label: labelHTML,
				value: value,
				id: this._getListBoxID(label, value),
				type: label
			};
			this._listBox.addEntry(entry);
		}
		
		this._contentSite.appendChild(this._listBox.domNode);
	},

	/*
 	 * @param String label
 	 * @param String value
 	 * 
 	 * @return String
 	 */	
	_getListBoxID: function(label, value) {
		return label + this._static._SEPARATOR + value;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */	
	_getIconMap: function() {

		var iconMap = {};

		if (!this.iconProvider) {
			return iconMap;
		}
		
		for (var index = 0; index < this.parameterNames.length; index++) {
			var name = this.parameterNames[index];
			iconMap[name] = this.iconProvider(this.parameterID, name);
		}
		
		return iconMap;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getListBoxHeight: function() {
		
		var targetSize;
		if (this.parameterNames.length < this._static._MIN_LIST_SIZE) {
			targetSize = this._static._MIN_LIST_SIZE;
		} else if (this.parameterNames.length > this._static._MAX_LIST_SIZE) {
			targetSize = this._static._MAX_LIST_SIZE;
		} else {
			targetSize = this.parameterNames.length;
		}
		
		var pixels = targetSize * this._static._LIST_ENTRY_HEIGHT;
		
		return new String(pixels) + "px";
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_connectDialogHandlers: function() {
		// OK button
		this._clickOKHandler = this.connect(this._okButton, "onclick", this._handleEventOKButtonClicked);
		// Cancel button
		this._clickCancelHandler = this.connect(this._cancelButton, "onclick", this._handleEventCancelButtonClicked);
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_disconnectDialogHandlers: function() {
		// OK button
		if (this._clickOKHandler) {
			this.disconnect(this._clickOKHandler);
			delete this._clickOKHandler;
		}
		// Cancel button
		if (this._clickCancelHandler) {
			this.disconnect(this._clickCancelHandler);
			delete this._clickCancelHandler;
		}
	},
	
	/*
 	 * @param Object entry
 	 * 
 	 * @return void
 	 */	
	_handleEventEntryDoubleClicked: function(entry) {
		this._handleEventOKButtonClicked(null);
	},	
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOKButtonClicked: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}

		// Get selection
		var selection = this._listBox.getSelection();
		
		if (selection.length === 1) {
			this._doIt = true;
			this._selectedParameterName = selection[0].type;
			this._selectedParameterValue = selection[0].value;
		} else {
			this._doIt = false;
		}
		
		this.hide();
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_handleEventCancelButtonClicked: function(event) {
		dojo.stopEvent(event);
		this._doIt = false;
		this.hide();
		
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;
		
		this._static._SEPARATOR = "~";
		
		this._static._MIN_LIST_SIZE = 3;
		this._static._MAX_LIST_SIZE = 15;
		this._static._LIST_ENTRY_HEIGHT = 20; // pixels
	}
	
});

})();
