/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.utils.StringUtils");

dojo.require("dojo.string");

(function() {
	
	function startsWith(string, value) {
		if (string.length < value.length) {
			return false;
		} else {
			return string.substring(0, value.length) == value;
		}
	}
	
	function utf8Encode(string) {
		string = string.replace(/\r\n/g, "\n");
		var utf8String = "";
		for (var n = 0; n < string.length; n++) {
			var c = string.charCodeAt(n);
			if (c < 128) {
				utf8String += String.fromCharCode(c);
			} else if ((c > 127) && (c < 2048)) {
				utf8String += String.fromCharCode((c >> 6) | 192);
				utf8String += String.fromCharCode((c & 63) | 128);
			} else {
				utf8String += String.fromCharCode((c >> 12) | 224);
				utf8String += String.fromCharCode(((c >> 6) & 63) | 128);
				utf8String += String.fromCharCode((c & 63) | 128);
			}
		}
		return utf8String;
	}
	
	function utf8Decode(utf8String) {
		var string = "";
		var i = 0;
		var c = c1 = c2 = 0;
		while (i < utf8String.length) {
			c = utf8String.charCodeAt(i);
			if (c < 128) {
				string += String.fromCharCode(c);
				i++;
			} else if ((c > 191) && (c < 224)) {
				c2 = utf8String.charCodeAt(i + 1);
				string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
				i += 2;
			} else {
				c2 = utf8String.charCodeAt(i + 1);
				c3 = utf8String.charCodeAt(i + 2);
				string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
				i += 3;
			}
		}
		return string;
	}
	
	function stringToHTML(string) {
		return string.replace(/[&<>]/g, 
			function(m){
				var map = {
					"&": "amp",
					"<": "lt",
					">": "gt"
				};
				return "&" + map[m] + ";";
			}
		);
	}
	
	function stringToDate(string) {
        string = dojo.string.trim(string);
        var a = string.split('/');
        var d = new Date();
        d.setTime(0);
        d.setDate(parseInt(a[0]));
        d.setMonth(parseInt(a[1]) - 1);
        d.setFullYear(parseInt(a[2]));
        return d;
	}

	function stripQuotes(string) {
		return string.replace(/^'(.*)'$/,'$1');
	}
	
	function splitValues(string) {
		var values = stripQuotes(string).split("','");
		var result = new Array();
		for (var i = 0; i < values.length; i++) {
			result.push("'" + values[i] + "'");
		}
		return result;
	}
	
	com.ibm.team.reports.web.ui.internal.utils.StringUtils.startsWith = startsWith;
	com.ibm.team.reports.web.ui.internal.utils.StringUtils.utf8Encode = utf8Encode;
	com.ibm.team.reports.web.ui.internal.utils.StringUtils.utf8Decode = utf8Decode;
	com.ibm.team.reports.web.ui.internal.utils.StringUtils.stringToHTML = stringToHTML;
	com.ibm.team.reports.web.ui.internal.utils.StringUtils.stringToDate = stringToDate;
	com.ibm.team.reports.web.ui.internal.utils.StringUtils.stripQuotes = stripQuotes;
	com.ibm.team.reports.web.ui.internal.utils.StringUtils.splitValues = splitValues;
})();

