/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * @author bjiang
 */

dojo.provide("com.ibm.team.scm.web.ui.internal.view.browse.FileDiffView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dijit._base.scroll");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function() { // module container

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ScmUtil					= com.ibm.team.scm.web.client.util;
var ScmClient				= com.ibm.team.scm.web.client.internal.ScmClient;
var	PlatformUI 				= net.jazz.ajax.ui.PlatformUI;
	
dojo.declare(
	"com.ibm.team.scm.web.ui.internal.view.browse.FileDiffView",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/view/browse/templates/FileDiffView.html"),	

		//Flags used to check for change set changes
		NONE: 0x00,
		ADD: 0x01,
		MODIFY: 0x02,
		RENAME: 0x04,
		REPARENT: 0x08,
		DELETE: 0x10,
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
			this.change = null;
		},
		
		postCreate: function() {
		},
		
		showWidget: function(change, changeSetDTO) {
			if(this.change && this.change.itemId == change.itemId)
				return;
				
			this.change = change;
			dojox.data.dom.removeChildren(this.diffDiv);
			dojox.data.dom.removeChildren(this.diffTableHead);
			dojox.data.dom.removeChildren(this.diffTableBody);
			
			
			var isNone = (change.changeType & this.NONE) != 0;
			var isAdded = (change.changeType & this.ADD) != 0;
			var isModified = (change.changeType & this.MODIFY) != 0;
			var isRenamed = (change.changeType & this.RENAME) != 0;
			var isReparented = (change.changeType & this.REPARENT) != 0;
			var isDeleted = (change.changeType & this.DELETE) != 0;
			
			var changeType = "";
			changeType = isAdded ? (changeType + this.messages.FileDiffView_Added + " ") : changeType;
			changeType = isModified ? (changeType + this.messages.FileDiffView_Modified + " ") : changeType;
			changeType = isRenamed ? (changeType + dojo.string.substitute(this.messages.FileDiffView_Renamed, [change.path]) + " ") : changeType;
			changeType = isReparented ? (changeType + this.messages.FileDiffView_Moved + " ") : changeType;
			changeType = isDeleted ? (changeType + this.messages.FileDiffView_Deleted + " ") : changeType;
			var itemChangedMsg = dojo.string.substitute(this.messages.FileDiffView_ItemWasChanged, [change.path, changeType]);
			this.diffDiv.appendChild(document.createTextNode(itemChangedMsg));

			//TODO: add work item associations
//			if(changeSetDTO.reasons && changeSetDTO.reasons.length > 0) { ScmUtil.createReasonElement(changeSet.reasons[i])	}
			
			if(isModified && change.diff) {
				this._showDiff(change, changeSetDTO);
			}
		},
			
		_showDiff: function(change, changeSetDTO) {	
				
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			var path = change.path;
			var index = path.lastIndexOf("/");
			path = index > -1 ? path.substring(index+1) : path;
			th.appendChild(document.createTextNode(dojo.string.substitute(this.messages.FileDiffView_DiffPath, [path])));
			tr.appendChild(th);
			this.diffTableHead.appendChild(tr);
			
			var span = document.createElement("span");
			dojo.addClass(span, "right");
			th.appendChild(span);

			var anchor= document.createElement("a");
			var workspaceItemId = (changeSetDTO.workspace != null)? changeSetDTO.workspace.itemId : null;
			anchor.href= ScmUtil.getViewChangeSetItemURI(workspaceItemId, changeSetDTO.component.itemId, change.itemType, change.itemId, change.beforeStateId);	
			anchor.title= dojo.string.substitute(this.messages.StreamWorkspaceView_19, [ change.path ]);
			var text= document.createTextNode(this.messages.StreamWorkspaceView_20);
			anchor.appendChild(text);
			span.appendChild(anchor);
			
			var text= document.createTextNode(this.messages.StreamWorkspaceView_21);
			span.appendChild(text);
			
			var anchor= document.createElement("a");
			anchor.href= ScmUtil.getViewChangeSetItemURI(workspaceItemId, changeSetDTO.component.itemId, change.itemType, change.itemId, change.afterStateId);	
			anchor.title= dojo.string.substitute(this.messages.StreamWorkspaceView_22, [ change.path ]);
			var text= document.createTextNode(this.messages.StreamWorkspaceView_23);
			anchor.appendChild(text);
			span.appendChild(anchor);
			
			var changes = ScmUtil.parseDiff(change.diff);
			for (var i = 0; i < changes.length; i++) {
				var tr = document.createElement("tr");
				var td = document.createElement("td");
				td.setAttribute("width", "100%");
				
				var div1 = document.createElement("div");
				div1.className = "diff-content";
				
				var count = 0;
				var lines = [];
				var cr = 0;
				var lf = 0;
				var crlf = 0;

				var j = 0;
				while (count < changes[i].length) {	
					//Check for the newline separator being used in the diff.
					if (cr != -1) {
						cr = changes[i].indexOf("\r", count);
					}
					if (lf != -1) {
						lf = changes[i].indexOf("\n", count);
					}
					if (crlf != -1) {
						crlf = changes[i].indexOf("\r\n", count);
					}
					
					//Extract the line in the diff.
					if (cr != -1 || lf != -1 || crlf != -1) {
						if (crlf != -1 && crlf <= cr && crlf < lf) {
							lines[j] = changes[i].slice(count, crlf);
							count = crlf + 2;
							j++;
						}
						else if(cr!=-1 && ((lf != -1 && cr < lf) || lf == -1)) {
							lines[j] = changes[i].slice(count, cr);
							count = cr + 1;
							j++;
						}
						else if(lf!=-1) {
							lines[j] = changes[i].slice(count, lf);
							count = lf + 1;
							j++;
						}
					}
					else {	//Last line in diff reached.
						lines[j] = changes[i].slice(count, changes[i].length);
						count = changes[i].length;
						j++;
					}
					
					var div = document.createElement("div");
					
					//Highlight the diff sections
					if (lines[j-1].charAt(0) == '@') {
						div.className = "diff3";
					}
					else if (lines[j-1].charAt(0) == '-') {
						div.className = "diff1";
					}
					else if (lines[j-1].charAt(0) == '+') {
						div.className = "diff2";
					}
					else if (lines[j-1]=="\\ No newline at end of file") {
						div.className = "diff4";
					}
					else {
						div.className = "diff";
					}
					var text = document.createTextNode(lines[j-1]);
					div.appendChild(text);
					div1.appendChild(div);
				}
				
				td.appendChild(div1);
				tr.appendChild(td);
				this.diffTableBody.appendChild(tr);
			}
		}
		
});

})();