/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

 // Use the jslint 'good things' default with the exception of: nomen: false (don't check variable names since dijit api uses _ which is against jslint rules) and forin
 /*jslint white: true, onevar: true, browser: true, undef: true, nomen: false, eqeqeq: true, plusplus: true, bitwise: true, regexp: true, strict: true, newcap: true, immed: true */
 /*global com, dijit, dojo, dojox, net, window */
 "use strict";
 
dojo.provide("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.TextEditorItem");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function () {
	/**
	 * Widget to edit text file content.
	 */
	dojo.declare(
			"com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.TextEditorItem", 
			[dijit._Widget, dijit._Templated, dijit._Contained], {
		templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/widgets/resourcesEditor/templates/TextEditorItem.html"),
		widgetsInTemplate : true,
		constructor: function (params) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
			this.labelEditContent = this.messages.TextEditorItem_labelEditContent;
		},
		postCreate: function () {
			var connection, that;
			that = this;
			// Don't enable editor until it is populated later
			this._editor.disabled = true;
			// Detect when user did an edit so parent editor can enable save button
			connection = dojo.connect(this._editor, "onkeypress", function (evt) {
				that.isDirty = true;
				dojo.disconnect(connection);
				that.getParent().getParent().onRefreshed();
			});
		},
		/**
		 * Download the text file's content and put it in the editor.
		 * If this is a file addition (argument null), does nothing. 
		 */
		resetContent: function (versionableDTO) {
			var headers, that, xhr, d;
			d = new dojo.Deferred();
			if (!versionableDTO) {
				/* Workaround for IE8 textarea display bug when width is set in percentage at creation time */
				this._editor.style.display = "block";
				dojo.addClass(this._editor, "com-ibm-team-scm-textEditorItem-textarea-fullwidth");
				this._editor.disabled = false;
				d.callback(true);
				return;
			}
			headers = {
				"accept-charset" : "utf-8"
			};
			that = this;
			xhr = dojo.xhrGet({
				url: "../../service/com.ibm.team.filesystem.common.IFileContentService/content/" + versionableDTO.versionable.itemId + "/" + versionableDTO.versionable.stateId,
				headers: headers,
				load: function (content) {
					/* Workaround for IE8 textarea display bug when width is set in percentage at creation time */
					that._editor.style.display = "block";
					dojo.addClass(that._editor, "com-ibm-team-scm-textEditorItem-textarea-fullwidth");
					that._editor.disabled = false;
					that._editor.value = content;
					d.callback(true);									
				},
                error: function (content) {
					d.errback(new Error(""));
                }
			});
			return d;
		}, 
		getText: function () {
			return this._editor.value;
		},
		isDirty: function () {
			return this.isDirty;
		}
	}
	);
}());