/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2008, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
 
dojo.provide("com.ibm.rational.cm.web.ui.page.CqLoginController");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("dojo.cookie");
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.view.common.ButtonDialog");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "cqLoginController");

(function(){

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var cq = com.ibm.rational.cm.web;
var BASE_URL = "cqlogin.cq";
var messages, _loginDlg, _loginDlgNoDb, _connDlg, _status;
var _processing, _queue, _needPing, _pinging, _isConnected, _errorMsg;
var ATTR_CONTENT = "content";
var PROP_LICENSE_REQUIRED = "CQ_BRIDGE_LICENSE_REQUIRED";
var DIALOG_WIDTH = 380;

var LoginStatus = {
	UNCHECKED: -1,
	AUTHENTICATED: 0,
	SESSION_TIMEOUT: 1
}

// summary: return client time zone in format: GMT+5:45 or GMT-7:00
// Date.getTimezoneOffset( ) returns the number of minutes difference 
// between the GMT or UTC time and the local time.
var _getTimeZoneOffset = function(){
	var tzOffset = (new Date).getTimezoneOffset();
	if(tzOffset == 0){
		return "";
	}
	var sign = "-";
	if(tzOffset < 0){
		sign = "+"
		tzOffset = -tzOffset;
	}
	var minutes = (tzOffset%60);
	if(minutes<10){ // server does not accept: GMT8:0, only allow GMT8:00
		minutes = "0" + minutes;
	}		
	var hour = Math.round(tzOffset/60);
	if(minutes>0){
		hour --;
	}
	tzOffset = "GMT" + sign + hour + ":" + minutes;
	return tzOffset; 		
}

var _updateErrorMessage = function(/* Object */ obj){
	// update error message in the login dialog
	var msg, welcomeDesc;
	if(obj){
		msg = obj.errorMsg;		
		if(obj.hideLogin){
			welcomeDesc = msg;
		}
	}
	var errMsgNode = dojo.byId("loginErrorMsgId");
	if(errMsgNode){
		errMsgNode.innerHTML = (msg && msg.length > 0) ? msg : "";
	}
	
	// notify CqPageView to update welcome message
	dojo.publish("CQ_WELCOME_MESSAGE", [{
		errorMsg: welcomeDesc
	}]);
}

dojo.declare(
	"com.ibm.rational.cm.web.ui.page.CqLoginController",
	null,
{
	COOKIE_NAME_FOR_DATABASE: "cqbridge_database",

	constructor: function(){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "cqLoginController");
		_status = LoginStatus.UNCHECKED;
		_processing = false; // true if it is in the middle of authenticating
		_queue = [];
		_loginInfo = {};
		_loginXhr = null;
		_needPing = true;
		_pinging = false;
		_isConnected = false;
	},
	
	setLoading: function(/* boolean */ loading, /* String */ message){
		var workbench = PlatformUI.getWorkbench();
		if(workbench){
			if(loading){
				workbench.showMessage(message);
			}else{
				workbench.hideMessageArea();
			}
		}
	},
	
	openLoginDialog: function(/* Object */ obj){
		if(!obj){
			obj = {};
		}
		var userdb = _loginInfo.userdb;
		if(userdb){
			var index = userdb.lastIndexOf("/");
			if(index >= 0){
				obj.repository = userdb.substring(0,index);
				obj.db = userdb.substring(index+1);
			}
		}
		if(_loginDlg || !obj.repository || obj.repository.length === 0 || !obj.db || obj.db.length === 0){
			this._openLoginDialogSelectDb(obj);
		}else{
			this._openLoginDialogWithoutDb(obj);
		}
	},
	
	/*
	 * Opens a login dialog with repository list dropdown
	 * HTML string is returned from the server, so create dialog using that
	 */
	_openLoginDialogSelectDb: function(/* Object */ obj){
		if(!_loginDlg){
			_loginDlg = new com.ibm.rational.cm.web.ui.view.common.Dialog({
				title: messages.LoginDlgTitle,
				iconClass: "welcomeIcon",
				content: messages.RetriveDBList
			});
			_loginDlg.resize({ w: DIALOG_WIDTH });
		}

		// when CQ server is not connected or no client access license for ClearQuest-bridge,
		// don't fetch repository list and just show error message
		var errMsg = obj.errorMsg;
		if(obj.hideLogin && (errMsg && errMsg.length > 0)){
			_loginDlg.attr(ATTR_CONTENT, '<div id="loginErrorMsgId" style="color:red;">'+errMsg+"</div>");
			_loginDlg.show();
			_updateErrorMessage(obj);
			return;
		}
		
		var _onDialogShow = function(){
			_updateErrorMessage(obj);
			
			// 79730, when session time out, don't show password value in login dialog
			// Never show password in login dialog
			if (dojo.byId("passwordId")) { dojo.byId("passwordId").value = ""; }
		
			// if repository is specified, set value and disable it
			var repo = obj.repository;
			if(repo && repo.length > 0){
				var dropDown = dijit.byId("repositoryListId");
				if(dropDown){
					dropDown.attr({
						value: repo,
						disabled: true
					})
				}
			}
		}
		
		if(!_loginDlg._init){
			// get dialog content HTML (form containing ID, password, repository list) 
			var self = this;
			cq.xhrGet({
				url: BASE_URL,
				onBegin: function(){
					_loginDlg.setLoading(true);
					self.setLoading(true, messages.RetriveDBList);
				},
				load: function(response, ioArgs){
					var startStr = '<table width="100%" align="center" border="0px"';
					
					// check if valid response
					if(response.indexOf(startStr) == -1){
						var obj = dojo.fromJson(response);
						if(obj && obj.status == "false"){
							var msg = obj.message;
							if(msg.MESSAGE){
								msg = msg.MESSAGE;
							}
							_loginDlg.attr(ATTR_CONTENT,"<div>"+messages.ProcessErr+"</div><div>"+msg+"</div>");
							return;
						}
					}
					
					// add FORM tag in order to support ENTER key to login
					response = response.replace(startStr, '<form method="POST" id="cqLoginForm">'+startStr);
					var endStr = '</table>';
					response = response.replace(endStr, endStr+'</form>'); // replace 1st TABLE end tag
					
					// CqValidationTextbox does not exist in RCM, so make it standard <INPUT type="text" />
					response = response.replace(/dojoType="cq.widget.CqValidationTextbox"/g, "");

					// when using FilteringSelect, dialog is on top of dropdown menu
					// so we explicitly set z-index of dropdown menu in CqPageView.css#widget_repositoryListId_dropdown
					dojo["require"]("dijit.form.FilteringSelect"); 
//					response = response.replace(/dojoType="dijit.form.FilteringSelect"/g, ""); // when using standard SELECT

					// this is not mandatory, but use standard <BUTTON> element instead of dijit.form.Button
					// (dijit.form.Button seems not to be used much in Jazz)
					response = response.replace(/dojoType="dijit.form.Button"/g, "");

					// remove button action
					response = response.replace("onclick='cqLoginService.submitLogin()'", 'type="submit"');
					response = response.replace("onclick='cqLoginService.cancelLogin()'", 'type="button"');

					// remove "Remember ID" checkbox
					var start = response.indexOf("<!--  remember login id -->");
					var end = response.indexOf("</tr>", start);
					response = response.substring(0, start) + response.substring(end+5);
					
					// 81852, remove colon in label
					response = response.replace(new RegExp("</label>:</td>","gm"), "</label></td>");

					// set onLoad handler
					_loginDlg.onLoad = function(){
						dojo.byId("loginId_label").innerHTML = messages.LoginId;
						dojo.byId("password_label").innerHTML = messages.Password;
						//dojo.byId("repos_label").innerHTML = messages.Repository; // when using standard SELECT
						dojo.byId("repositoryListId_label").innerHTML = messages.Repository; // when using FilteringSelect
						
						// show dropdown when input is clicked
						var dropDown = dijit.byId("repositoryListId");
						dropDown.connect(dropDown.textbox, "onclick", function(){
							if(dropDown._isShowingNow){
								dropDown._hideResultList();
							}else{
								dropDown._startSearch("");
							}
						});
						
						// setup login button
						var doLogin = function(){
							_loginDlg.setLoading(true);
							self.doLogin({
								content: {
									action: "DoLogin",
									loginId: dojo.byId("loginId_Id").value,
									password: dojo.byId("passwordId").value,
									repository: dojo.byId("repositoryListId").value,
									tzOffset: _getTimeZoneOffset()
								},
								onLoginSuccess: dojo.hitch(_loginDlg, "hide"),
								handle: function(response){
									// if this is a session timeout
									// don't show connect dialog and use the loginInfo value
									if(_status == LoginStatus.SESSION_TIMEOUT){
										var userdb = _loginInfo.userdb;
										if(userdb){
											var index = userdb.lastIndexOf("/");
											if(index >= 0){
												self.doLogin({
													content: {
														action: "DoConnect",
														userDb: userdb.substring(index+1)
													},
													onLoginSuccess: dojo.hitch(_loginDlg, "hide"),
													handle: function(){
														_loginDlg.setLoading(false);
													}
												});
												return;
											}
										}
									}	
									
									// if there are more than one userDb,
									// HTML string will be returned, so show connect dialog
									_loginDlg.setLoading(false);
									if(response.indexOf("<table") != -1){
										_loginDlg.hide();
										self._openConnectDialog(response);
									}
								}
							});
						}
						
						var connectBtn = dojo.byId("loginButtonId");
						connectBtn.innerHTML = messages.Connect;
						_loginDlg.connect(connectBtn, "onclick", doLogin);
						
						_loginDlg.connect(dojo.byId("cqLoginForm"), "onsubmit", function(e){
							dojo.stopEvent(e);
							doLogin();
						});
						
						// setup cancel button
						var cancelBtn = dojo.byId("loginCancelButtonId")
						cancelBtn.innerHTML = messages.Cancel;
						_loginDlg.connect(cancelBtn, "onclick", "onCancel");
						
						// setup cancel handler (called in both cancel button and dialog close)
						_loginDlg.connect(_loginDlg, "onCancel", function(){
							self._handleCancel();
						});
						
						_onDialogShow();
						
						// set focus to User ID field
						dijit.focus(dojo.byId("loginId_Id"));
					}
					_loginDlg.attr(ATTR_CONTENT, response);
					_loginDlg._init = true;
				},
				error: function(response, ioArgs){
					if(response.status == 500){
						var start = response.responseText.indexOf("<pre>");
						var end = response.responseText.indexOf("</pre>", start);
						var serverResponse = response.responseText.substring(start + 5, end);
						_loginDlg.attr(ATTR_CONTENT,"<div>"+messages.ProcessErr+"</div><div>"+serverResponse+"</div>");
					}else{
						_loginDlg.attr(ATTR_CONTENT,"<div>"+messages.ProcessErr+"</div><div>"+response+"</div>");
					}
				},
				handle: function(response, ioArgs){
					_loginDlg.setLoading(false);
					self.setLoading(false);
				},
				preventCache: true,
				noLogin: true, // custom argument in cq.xhr
				handleAs: "text"
			});
		}
		
		_onDialogShow();
		
		_loginDlg.show();
	},
	
	/*
	 * Opens a login dialog without repository list dropdown
	 */
	_openLoginDialogWithoutDb: function(/* Object */ obj){
		if(!_loginDlgNoDb){
			_loginDlgNoDb = new com.ibm.rational.cm.web.ui.view.common.Dialog({
				id: "cqLoginDialog",
				title: messages.LoginDlgTitle,
				iconClass: "welcomeIcon"
			});
			_loginDlgNoDb.resize({ w: DIALOG_WIDTH });
			
			var repository = obj.repository;
			var db = obj.db;
			var content = dojo.string.substitute('<form method="POST" id="cqLoginForm"><div id="loginErrorMsgId" colspan="2"></div><table id="loginTbl"><tbody><tr><th><label id="loginId_label" for="loginId_Id">${LoginId}</label></th><td><input id="loginId_Id" type="text" name="loginId_Id" /></td></tr><tr><th><label id="password_label" for="passwordId">${Password}</label></th><td><input id="passwordId" type="password" name="passwordId" /></td></tr><tr><th>${repoAndDb}</th><td style="font-weight:bold;">'+repository+'/'+db+'</td></tr><tr><td></td><td style="height:30px;"><button id="loginButtonId" type="submit">${LogIn}</button><button id="loginCancelButtonId" type="button">${Cancel}</button></td></tr></tbody></table></form>', messages);
			_loginDlgNoDb.setContent(content);
			
			var self = this;
			var doLogin = function(){
				_loginDlgNoDb.setLoading(true);
				self.doLogin({
					content: {
						action: "DoLogin",
						loginId: dojo.byId("loginId_Id").value,
						password: dojo.byId("passwordId").value,
						repository: repository,
						tzOffset: _getTimeZoneOffset()
					},
					onLoginSuccess: function(){
						_loginDlgNoDb.hide();
					},
					handle: function(response){
						_loginDlgNoDb.setLoading(false);

						// if there are more than one userDb,
						// HTML string will be returned, so connect silently without showing connect dialog
						if(response.indexOf("<table") != -1){
							_loginDlgNoDb.setLoading(true);
							self.doLogin({
								content: {
									action: "DoConnect",
									userDb: db
								},
								onLoginSuccess: dojo.hitch(_loginDlgNoDb, "hide"),
								handle: function(){
									_loginDlgNoDb.setLoading(false);
								}
							});
						}
					}
				});
			}
			_loginDlgNoDb.execute = doLogin;
			
			// setup cancel handler (called in both cancel button and dialog close)
			_loginDlgNoDb.connect(dojo.byId("loginCancelButtonId"), "onclick", function(){
				self._handleCancel();
				_loginDlgNoDb.hide();
			});

			_loginDlgNoDb.connect(dojo.byId("cqLoginForm"), "onsubmit", function(e){
				dojo.stopEvent(e);
				doLogin();
			});

			_loginDlgNoDb.startup();
		}
		
		// if hideLogin=true, hide login form table, and show error message
		dojo.byId("loginTbl").style.display = (obj.hideLogin) ? "none" : "";
		_updateErrorMessage(obj);
		
		// 79730, Never show password in login dialog
		if (dojo.byId("passwordId")) { dojo.byId("passwordId").value = ""; }
		
		_loginDlgNoDb.show();
	},
	
	/*
	 * args is a Object with
	 * {
	 * 	onLoginSuccess:		Function
	 *	handle:		Function
	 * }
	 */
	doLogin: function(args){
		// if login is already processing, return
		if(_processing){ return; }
		
		if(dojo.cookie.isSupported()){
			dojo.cookie(this.COOKIE_NAME_FOR_DATABASE, null, {expires: -1});
		}
		
		var self = this;
		var content = args.content;
		cq.xhrPost({
			url: BASE_URL,
			content: content,
			onBegin: function(){
				_processing = true;
				self.setLoading(true, messages.LoggingIn);
			},
			processDeferred: function(dfd){
				_loginXhr = dfd;
			},
			load: function(response, ioArgs){
				_processing = false;
				try{
					var loginInfo = dojo.fromJson(response);
				 	// there is only one user DB
					if(loginInfo){
						if(loginInfo.status == "true"){
							_loginInfo = loginInfo;
							if(dojo.cookie.isSupported()){
								dojo.cookie(self.COOKIE_NAME_FOR_DATABASE, loginInfo.userdb, { path: net.jazz.ajax._contextRoot });
							}
							self.onLoginSuccess();
							cq.executeFunc(args.onLoginSuccess);
						}else if(loginInfo.status == "false"){
							var msg = loginInfo.message;
							if(msg && msg.length > 0){
								self.openLoginDialog({
									errorMsg: msg
								});
							}
						}
					}
				}catch(e){
				}
			},
			error: function(response, ioArgs){
				_processing = false;
				self.openLoginDialog({
					errorMsg: response
				});
			},
			handle: function(response, ioArgs){
				_loginXhr = null;
				self.setLoading(false);
				
				var handle = args.handle;
				if(handle && dojo.isFunction(handle)){
					handle(response);
				}
			},
			noLogin: true, // custom argument in cq.xhr
			handleAs: "text"
		});		
	},
	
	// Opens Connect dialog if there are multiple user DBs
	// response : HTML string of connect dialog content
	_openConnectDialog: function(response){
		if(!_connDlg){
			// drop down menu is below dialog for filtering select, so use standard SELECT element
			response = response.replace(/dojoType="dijit.form.ComboBox"/g, "");
			// this is not mandatory, but use standard <BUTTON> element instead of dijit.form.Button
			// (dijit.form.Button seems not to be used much in Jazz)		
			response = response.replace(/dojoType="dijit.form.Button"/g, "");
			// remove button action
			response = response.replace("onclick='cqLoginService.submitConnect()'", "");
			response = response.replace("onclick='cqLoginService.cancelConnect()'", "");
			// 81852, remove colon in label
			response = response.replace("</label>:</td>", "</label></td>");
		
			var self = this;
			_connDlg = new com.ibm.rational.cm.web.ui.view.common.Dialog({
				title: messages.LoginDlgTitle,
				iconClass: "welcomeIcon",
				content: messages.Loading
			});
			_connDlg.resize({ w: 350 });
			_connDlg.onLoad = function(){
				// setup labels
				dojo.byId("selectDbHeader").innerHTML = messages.DBSelection;
				dojo.byId("userDbList_label").innerHTML = messages.Database;
				
				// setup submit button
				_connDlg.connect(dojo.byId("cqConnectSubmitButtonId"), "onclick", function(){
					_connDlg.setLoading(true);
					self.doLogin({
						content: {
							action: "DoConnect",
							userDb: dojo.byId("userDbListId").value
						},
						handle: function(){
							_connDlg.setLoading(false);
							_connDlg.hide();
						}
					});					
				});
				
				// setup cancel button
				_connDlg.connect(dojo.byId("cqConnectCancelButtonId"), "onclick", "onCancel");
				
				// setup cancel handler (called in both cancel button and dialog close)
				_connDlg.connect(_connDlg, "onCancel", function(){
					self._handleCancel();
				});
			}
			_connDlg.attr(ATTR_CONTENT, response);
		}
		_connDlg.show();
	},
	
	onLoginSuccess: function(){
		_status = LoginStatus.AUTHENTICATED;
					
		// processes callback functions
		dojo.forEach(_queue, function(obj){
			cq.executeFunc(obj.callback);
		});
		
		// clear the queue
		_queue = [];
		
		dojo.publish("CQ_LOGIN_SUCCESS");
	},

	_handleCancel: function(){
		// execute onCancel handler
		dojo.forEach(_queue, function(obj){
			cq.executeFunc(obj.onCancel);
		});
		
		// clear the queue
		_queue = [];
		
		// cancel login/connect request
		if(_loginXhr){
			cq.executeFunc(dojo.hitch(_loginXhr, "cancel"));
		}
		
		// notify CqPageView to update welcome message
		dojo.publish("CQ_WELCOME_MESSAGE");
	},
	
	isConnected: function() {
		return _isConnected;
	},
	
	isAuthenticated: function(){
		return (_status == LoginStatus.AUTHENTICATED);
	},
	
	/*
	 * Set login status to SESSION_TIMEOUT
	 */
	setSessionTimout: function(){
		_status = LoginStatus.SESSION_TIMEOUT;
	},
	
	getLoginInfo: function(key){
		return _loginInfo[key];
	},

	/*
	 * argument should be form of "repository/userdb" 
	 */
	setDBInfo: function(value){
		_loginInfo["userdb"] = value;
	},
	
	/*
	 * Return true if the passed item is mastered locally
	 */
	isLocallyMastered: function(/* Object */ item){
		var localReplica = this.getLoginInfo("localReplica");
		var masterReplica = item.masterReplica;		
		// The passed item is locally mastered if one of the conditions below is met
		return masterReplica == null ||
			masterReplica == "<local>" || 
			masterReplica == "<unknown>" ||
			masterReplica == localReplica;
		
	},
	
	/*
	 * Processes callback function immediately if already authenticated,
	 * and prompts for login dialog if not and processes callback function after authenticated
	 * 
	 * argument: JavaScript object in the following format
	 * 	{
	 * 		callback: function(){...},
	 * 		onCancel: function(){...},
	 * 		noPrompt: true/false (if true, don't prompt login dialog even if not authenticated. Call onCancel)
	 * 	}
	 */
	process: function(obj){
		if(this.isAuthenticated()){ // if already authenticated, processes callback function immediately
			return cq.executeFunc(obj.callback);
		}else{
			// add to queue
			_queue.push(obj);
			
			var self = this;
			var processLogin = function(prop){
				// if not authenticated and is not in the middle of login, so explicitly call login
				// otherwise, it is in middle of login and callback is called after successful login
				if(_processing){ return; }
				if(obj.noPrompt == true){
					cq.executeFunc(obj.onCancel);
				}else{
					if(!prop){
						prop = {};
					}
					
					// if no CAL, show corresponding error message
					if(_loginInfo[PROP_LICENSE_REQUIRED]){
						prop.errorMsg = messages.LicenseErr;
						prop.hideLogin = true;
					}else if(_status == LoginStatus.SESSION_TIMEOUT){
						prop.errorMsg = messages.SessionTimeoutErr;
					}

					// actually open login dialog
					self.openLoginDialog(prop);
				}				
			}
			
			// call ping if Ping has not been called yet
			if(_needPing){
				cq.xhrGet({
					url: BASE_URL+"?action=Ping",
					onBegin: function(){
						_needPing = false;
						_pinging = true;
						self.setLoading(true, messages.Loading);
					},
					load: dojo.hitch(this, function(response, ioArgs){
						if(!response || !dojo.isObject(response) || response.status == false){ // cannot connect to CQ server
							var	serverResponse = (dojo.isObject(response)) ? response.MESSAGE : cq.getErrStrFromResponseTxt(response);
							_errorMsg = '<div>'+serverResponse+'</div><div>'+messages.CqServerNotConnected+'</div>';
							processLogin({
								errorMsg: _errorMsg,
								hideLogin: true
							});
						}else{
							// store login info
							dojo.forEach([
								"userdb", "isFullTextSearchEnabled", "localReplica", PROP_LICENSE_REQUIRED
							], function(key){
								if(response[key]){
									_loginInfo[key] = response[key];
								}
							});
							
							if(response.status == "LOGIN_FAIL"){ // not authenticated yet or session timeout, so prompt for login
								if(dojo.cookie.isSupported()){
									dojo.cookie(self.COOKIE_NAME_FOR_DATABASE, null, {expires: -1});
								}

								processLogin();
							}else{ // CQ session is already established, so don't prompt for login
							
								// Get userdb from cookie since Ping action always returns userdb set in admin config instead of actual logged-in userdb
								if(dojo.cookie.isSupported()){
									var dbInCookie = dojo.cookie(self.COOKIE_NAME_FOR_DATABASE);
									if(dbInCookie && dbInCookie.length > 0){
										_loginInfo["userdb"] = dbInCookie;
									}
								}
								
								this.onLoginSuccess();
							}
							_isConnected = true;
						}
					}),
					error: function(response, ioArgs){
						var serverResponse = cq.getErrStrFromResponseTxt(response.responseText);
						if(!serverResponse){
							serverResponse = response;
						}
						_errorMsg = '<div>'+serverResponse+'</div><div>'+messages.CqServerNotConnected+'</div>';
						processLogin({
							errorMsg: _errorMsg,
							hideLogin: true
						});
					},
					handle: function(response, ioArgs){
						_pinging = false;
						self.setLoading(false);
					},
					noLogin: true
				});
			}else{
				if(!_pinging){
					if(_errorMsg){
						processLogin({
							errorMsg: _errorMsg,
							hideLogin: true
						});
					}else{
						processLogin();
					}
				}
			}
		}
		return null;
	},
	
	doLogout: function(callback){
		if(dojo.cookie.isSupported()){
			dojo.cookie(this.COOKIE_NAME_FOR_DATABASE, null, {expires: -1});
		}
		var self = this;
		cq.xhrPost({
			url: BASE_URL,
			content: { action: "DoLogout" },
			load: function(response, ioArgs){
				cq.executeFunc(callback);
				_status = LoginStatus.UNCHECKED;
				_loginInfo = {};
				_queue = []; // clear the queue
			},
			error: function(response, ioArgs){
				alert(messages.LogoutFail);
			},
			noLogin: true, // custom argument in cq.xhr
			handleAs: "text"
		});	
	}
});

})();

if(!com.ibm.rational.cm.cqLoginController)
	com.ibm.rational.cm.cqLoginController = new com.ibm.rational.cm.web.ui.page.CqLoginController();