/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

/*
 * Extension of dijit.Tree with selection function
 */
dojo.provide("com.ibm.rational.cm.web.ui.view.common.Tree");

dojo.require("dijit.Tree");

(function(){

var CLS_SELECTED = "selected";
var CLS_LOADING = "cqTreeLoading";

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.TreeNode",
	dijit._TreeNode,
{
	_setExpando: function(/*Boolean*/ processing){
		this.inherited(arguments);
		
		if(this.isExpandable){
			dojo.toggleClass(this.rowNode, CLS_LOADING, (processing === true));
		}
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.Tree",
	dijit.Tree,
{
	multiSelect: false,
	
	// JavaScript object
	// {
	//	type: String|Array (takes values "LIST"|"CHART"|"REPORT"|"REPORT-FORMAT")
	//	recordType: String
	// }
	filter: null,
	
	postCreate: function(){
		dojo.addClass(this.domNode, "com-ibm-rcm-Tree");
		this.inherited(arguments);
		
		// set node unchecked (for cancelling re-authentication after session timeout)
		var model = this.model;
		if(model && model.setUnchecked){
			var self = this;
			this.connect(model, "setUnchecked", function(item){
				var identity = model.getIdentity(item);
				if(!identity){ return; }
				var node = self._itemNodeMap[identity];
				if(node){
					node.state = "UNCHECKED";
					node.unmarkProcessing();
				}
			});
		}
		
		this._selected = [];
		this.connect(this, "focusNode", "setSelection");
		
		// enable filtering of tree nodes
		var filter = this.filter;
		if(filter){
			dojo.mixin(this, com.ibm.rational.cm.web.ui.view.common._TreeFilterMixin);
			if(dojo.isString(filter)){
				try{
					filter = this.filter = dojo.fromJson(filter);
				}catch(e){
					console.error("Specified filter value is not valid JSON object");
				}
			}
			this.setFilter(filter);
		}
	},
	
	// selection-related methods
	
	_onClick: function(/*Event*/ e){
		this._isCtrlKey = (e.ctrlKey);
		this.inherited(arguments);
	},
	
	_onKeyPress: function(/*Event*/ e){
		this._isCtrlKey = (e.ctrlKey);
		this.inherited(arguments);
	},
	
	setSelection: function(/* _tree.Node */ node){
		var isCtrlKey = this._isCtrlKey;
		var multiSelect = this.multiSelect;
		var isSelectionChanged = false;
		
		// set as new selection
		if(multiSelect && isCtrlKey){
			var index = dojo.indexOf(this._selected, node);
			if(index != -1){
				dojo.removeClass(this._selected[index].rowNode, CLS_SELECTED);
				this._selected.splice(index, 1);
			}else{
				this._selected.push(node);
				dojo.addClass(node.rowNode, CLS_SELECTED);
			}
			
			// selection always changes for multi-selection
			isSelectionChanged = true;
		}else{
			// check if newly selected node is the same or not
			var len = this._selected.length;
			if(len > 0){
				isSelectionChanged = !(len == 1 && node == this._selected[0]);

				// deselect previously selected nodes
				if(isSelectionChanged){
					this.clearSelection();
				}
			}else{ // no previous selection
				isSelectionChanged = true;
			}
			
			if(isSelectionChanged){
				this._selected = [node];
				dojo.addClass(node.rowNode, CLS_SELECTED);
			}
		}
		
		if(isSelectionChanged){
			this.onSelectionChanged(this._selected);
		}
	},
	
	clearSelection: function(){
		for(var i = this._selected.length - 1; i >= 0; i--){
			dojo.removeClass(this._selected[i].rowNode, CLS_SELECTED);
			this._selected.splice(i, 1);
		}
	},
	
	/*
	 * User-overridable function to notify selection has changed
	 */
	onSelectionChanged: function(/* Array */ selected){},
	
	getSelection: function(){
		return this._selected;
	},
	
	// override to use com.ibm.rational.cm.web.ui.view.navigation.NavigatorNode
	_createTreeNode: function(/*Object*/ args){
		return new com.ibm.rational.cm.web.ui.view.common.TreeNode(args);
	}
});

/*
 * Extension of tree which has filtering capability
 */
com.ibm.rational.cm.web.ui.view.common._TreeFilterMixin = {
	
	setFilter: function(filter){
		this.filter = filter;

		// show/hide nodes from root
		if(this.rootNode){
			this._updateChildDisplay(this.rootNode);
		}
	},
	
	/*
	 * Show/hide child nodes recursively
	 */
	_updateChildDisplay: function(/*TreeNode*/ node){
		dojo.forEach(node.getChildren(), function(child){
			this._filterNode(child);
			this._updateChildDisplay(child);
		}, this);
	},
	
	/*
	 * Show/hide tree node depending on filter value
	 */
	_filterNode: function(/*TreeNode*/ node){
		var store = this.model.store;
		// always show folders
		// don't use node.isExpandable since node.isExpandable=false for empty folders
		if(store.getValue(node.item, "type") == "FOLDER"){ return; }

		var bShow = true;
		for(var key in this.filter){
			var filterValue = this.filter[key];
			if(filterValue){
				var itemValue = store.getValue(node.item, key);
				if(dojo.isArray(filterValue)){
					// hide node if the item value match none of the items in array
					var ret = dojo.some(filterValue, function(item){
						return (itemValue == item);
					})
					if(!ret){
						bShow = false;
						break;
					}
				}else{
					if(itemValue != filterValue){ // hide node if the item value differs from specified filter
						bShow = false;
						break;
					}
				}
			}
		}
		node.domNode.style.display = (bShow) ? "" : "none"; // show node
	},
	
	// override dijit.Tree
	onOpen: function(/* dojo.data */ item, /*TreeNode*/ node){
		dojo.forEach(node.getChildren(), dojo.hitch(this, "_filterNode"));
	}
}

})();