/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui2.internal.widget.AbstractEditor");


dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxy");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("jazz.core.internal.loader");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory");

(function(){
    var WorkItemProxy = com.ibm.team.workitem.web.cache.internal.WorkItemProxy;
    var LayoutUtil = com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil;
    var Loader = jazz.core.internal.loader;
    var getClientCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    var WorkItemUtil = com.ibm.team.workitem.web.client.util;
    var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    var checkNonNegative = net.jazz.ajax.toolbox.lang.checkNonNegative;
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
    
  
    dojo.declare("com.ibm.team.workitem.web.ui2.internal.widget.AbstractEditor", [dijit._Widget, dijit._Templated], {
        constructor: function(workItemPageView){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
            this.pageController = this;
            this._childWidgets = {};
			this._eventHandles = [];
            this.isDirty = false;
        },
        destroy: function(isClose){
        	if(this.isDestroyed) return;
            if (this.workingCopy) {
				/* Remove all listeners and children */
				this.workingCopy.removeAllListeners({listener: this, parentSelector: "parentWidget"});
				this.workingCopy.removeAllListeners({listener: this, parentSelector: "parentController"});
				if(!this.isDestroyed) {
				   this.workingCopy.decrementUseCount(/* remove if zero */ isClose);
				}
            }
			for(var i=0;i<this._eventHandles.length;i++){
				dojo.unsubscribe(this._eventHandles[i]);
			}
			this.resetChildWidgets();
			this.isDestroyed = true;
			this.inherited(arguments);
        },
        isGuest: function(){
           return com.ibm.team.repository.web.client.session.isGuest();
        },
        focus: function() {
        	
        },
        getPropertyMap: function(propertyObject, originalObject){
            var properties;
            if (originalObject) {
                properties = originalObject;
            }
            else {
                properties = {}
            }
			if(!propertyObject || !propertyObject.properties) return properties;
            dojo.forEach(propertyObject.properties, function(property){
                //				if(properties[property.key]) {
                //					console.debug("**WARNING**: property " + property.key + " was replaced in LayoutUtil.getPropertyMap.");
                //				}
                properties[property.key] = property.value;
            })
            return properties;
        },
		getPropertyMapFromArray: function(parms /* keySelector, array, result */) {
			parms.keySelector = parms.keySelector || "id";
			parms.result = parms.result || {};
			parms.array = parms.array || [];
			
			for(var i=0;i<parms.array.length;i++) {
				if(parms.array[i]) {
					parms.result[parms.array[i][parms.keySelector]]=parms.array[i];
				}	
			}
			return parms;
		},
        getCurrentProjectArea: function(){
            return getClientCurrentProjectArea();
        },
		/* @deprecated Use fetchWorkItem */
 		fetchExistingWorkItem: function(id, projectArea, isForced, callback, isInit, isQuiet){
			var args = {};
			if(id) args.id = id;
			if(projectArea) args.projectArea = projectArea;
			if(isForced) args.isForced = isForced;
			if(callback) args.callback = callback;
			if(isInit) args.isInit = isInit;
			if(isQuiet) args.isQuiet = isQuiet;
			return this.fetchWorkItem(args);
		},
        fetchWorkItem: function(parms) {
//				id, projectArea, isForced, callback, isInit, isQuiet, isNew
            checkNotNull(parms.id, "id-fetchWorkItem");
            checkNotNull(parms.projectArea, "projectArea");
            checkNotNull(parms.callback, "calback");
			parms.isQuiet = parms.isQuiet== null ? false : parms.isQuiet;
			parms.isInit = parms.isInit == null ? false : parms.isInit;
			parms.isNew = parms.isNew || parms.id < 0;
			parms.fetchEditorPresentation = false;
			parms.fetchInlineEditorPresentation = true;
			// Validate that the id is a number and not new
			parms.id = parseInt(parms.id, 10);
            if (!parms.id) {
                this.handleError({
                    message: this.messages.errorOpeningWorkItem
                }, dojo.string.substitute(this.messages.errorOpeningWorkItemMessage, [("" + parms.id)]));
            }

			var operationMsg = dojo.string.substitute(this.messages.busyLoading, ["" + parms.id]);
            // isForced means to refresh from the server even if it is in the cache
			//  else use the working copy if available
            if (parms.isForced) {
				var operationMsg = dojo.string.substitute(this.messages.busyLoading, ["" + parms.id]);
			}
			else {
				// get this.workingCopy if it exists
				if (this._getThisWorkingCopy(parms.id, /*createIfNeeded*/ parms.isNew)) {
					if(this.workingCopy.isFetchedWithPresentation(parms)) {
						parms.callback(parms);
						return;
					}
				}
			}
			parms.operationMsg = operationMsg;
			parms.onSuccess = parms.callback;
			var workingCopy = this._getThisWorkingCopy(parms.id, /*createIfNeeded*/ true);
			
			if(workingCopy.isFetched && !parms.isForced) {
				parms.fetchWorkItem = false;
				workingCopy.fetchPresentations(parms);
			} else {
//				parms.fetchWorkItem = true;
				workingCopy.fetchWorkItem(parms);
			}
			
		},
		_getThisWorkingCopy: function(id, createIfNeeded) {
			if (!this.workingCopy) {
				// First see if there is already a work item in the cache
				var workingCopy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id);
				if (workingCopy) {
					this.workingCopy = workingCopy;
				} else {
					if(createIfNeeded) {
						this.workingCopy = this._createThisWorkingCopy(id);
					} else {
						return null;
					}
				}
            	this.workingCopy.incrementUseCount();
				this.setIsDirty(this.workingCopy.isChanged());
			}
			return this.workingCopy;
		},
		_createThisWorkingCopy: function(id) {
			return com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory.getWorkItemProxy({
				id: id,
				createIfNeeded: true
			});	
		},
		/*
		 * Before the work item is saved, get the widgetFormElements and widgetCommands
		 */
		_workItemAboutToSave: function(parms /* widgetFormElements, widgetCommands */) {
						
            for (var widget in this._childWidgets) {
                if (this._childWidgets[widget].getFormElementValues) {
                    this._childWidgets[widget].getFormElementValues(parms.widgetFormElements);
                }
            }
			
            for (var partName in this._childWidgets) {
                if (this._childWidgets[partName].getSaveCommands) {
                    this._childWidgets[partName].getSaveCommands(parms.widgetCommands);
                }
            }
			
		},
        storeWorkItem: function(successCallback, failureCallback, parms){
			this.workingCopy.storeWorkItem({
				instigator: this,
				resultParameters: parms || {}, 
				operationMsg: this.messages.busySaving,
				onSuccess: successCallback,
				onError: failureCallback
			});
        },
        /**
         * Returns <code>true</code> if the work item editor is displaying a new
         * work item, <code>false</code> if the work item editor is displaying an
         * existing work item.
         */
        isNewWorkItem: function(){
			return this.workingCopy.isNewWorkItem();
//            if (!this.workingCopy.storedObject) 
//                return false;
//            return this.workingCopy.storedObject.id < 0;
        },
        _hasValue: function(array, value){
            for (var i = 0; i < array.length; i++) {
                if (array[i] == value) {
                    return true;
                }
            }
            return false;
        },
        
        
        _getPresentationWidgetCreationSpec: function(presentation){
            var key = presentation.widget.replace(/\./g, "_");
            
						/*
			 * If the attribute has a specific widget, then map the key
			 */
			if(LayoutUtil.attributeSpecificWidgetMap[presentation.attributeId] 
				&& key == LayoutUtil.attributeSpecificWidgetMap[presentation.attributeId].fromKey ) {
				key = LayoutUtil.attributeSpecificWidgetMap[presentation.attributeId].toKey;
				presentation.widget = LayoutUtil.attributeSpecificWidgetMap[presentation.attributeId].toWidget
			}
			
            var result = {
                widgetName: presentation.widget,
                isMissingWidget: false
            };
            
            if (LayoutUtil.genericWidgetMap[key]) {
                result.originalWidgetName = presentation.widget;
                result.widgetName = LayoutUtil.genericWidgetMap[key].widgetName;
                if (LayoutUtil.genericWidgetMap[key].validationRegex) 
                    result.validationRegex = LayoutUtil.genericWidgetMap[key].validationRegex;
                if (LayoutUtil.genericWidgetMap[key].validationMessage) 
                    result.validationMessage = this.messages[LayoutUtil.genericWidgetMap[key].validationMessage];
                if (LayoutUtil.genericWidgetMap[key].isNumber) 
                    result.isNumber = LayoutUtil.genericWidgetMap[key].isNumber ? true : false;
                if (LayoutUtil.genericWidgetMap[key].modifiers) 
                    result.modifiers = LayoutUtil.genericWidgetMap[key].modifiers;
                key = LayoutUtil.genericWidgetMap[key].widgetName.replace(/\./g, "_");
            }
            
            if (!LayoutUtil.loadedWidgetMap[key]) {
                /*
                 * If it is not loaded and not missing, attempt to load it
                 */
                var loadedWidget;
                if (!LayoutUtil.missingWidgetMap[key]) {
                    try {
                        Loader.load_sync(presentation.widget);
                        loadedWidget = eval(presentation.widget);
                    } 
                    catch (e) {
                        console.debug("WorkItemEditor could not load: " + presentation.widget);
                    }
                }
                
                if (loadedWidget) {
                    result.widget = loadedWidget;
                    LayoutUtil.loadedWidgetMap[key] = loadedWidget;
                }
                else {
                    LayoutUtil.missingWidgetMap[key] = presentation.widget;
                    result.widget = com.ibm.team.workitem.web.ui.internal.view.layout.SeparatorWidget;
                    result.isMissingWidget = true;
                }
                
            }
            else {
                result.widget = LayoutUtil.loadedWidgetMap[key];
            }
            
            // This doesn't work because the widget is a prototype and not an object			
            //			if(result.widget.displaySpec) {
            //				this.mergeObjects(displaySpec, result.widget.displaySpec);	
            //			}
            
            return result;
        },
        /*
         * Update the header validation if it exists (to be overridden)
         */
		updateHeaderValidation: function() {
		},
		getAttachmentInfo: function(number ) {
			return this.workingCopy.getAttachmentInfo(number);
		},
        getWidget: function(workItem, presentation, valueNode, values, possibleValues, modifiers){
        
            var creationSpec = this._getPresentationWidgetCreationSpec(presentation);
            
            var widgetClass = creationSpec.widget;
            
            var propertyModifiers = LayoutUtil.getPropertyMap(presentation);
            modifiers = LayoutUtil.mergeObjects(modifiers || {}, propertyModifiers);
            
            var self = this;
            if (widgetClass) {
                var attributeId = propertyModifiers.attributeId != null ? propertyModifiers.attributeId : presentation.attributeId;
                attributeId = attributeId != null ? attributeId : presentation.widget.replace(/\./g, "_");
                var creationArgs = {
                    parentWidget: this,
                    creationSpec: creationSpec,
                    presentation: presentation,
                    workItem: workItem,
                    workingCopy: this.workingCopy,
                    parentController: this,
                    attributeLabel: presentation.attributeLabel || "Attribute",
                    attributeId: presentation.attributeId,
                    attributeValue: this.workingCopy.getValue({
                        path: ["attributes", attributeId]
                    }),
                    values: values,
                    valueNode: valueNode,
                    _modifiers: modifiers ||
                    {},
                    _possibleValues: possibleValues
                }
                
             // The header is a special case since it claims to be the parentController instead of the editor
                if(this.isHeader) {
                    creationArgs.headerParentController= this.parentController;
                }
                
                // If the widget is missing and we are not debugging, skip the widget and label
                if (creationSpec.isMissingWidget && !WorkItemUtil.isDebugMode()) 
                    return;
                
                if (creationSpec.isMissingWidget) {
                    creationArgs.widgetName = creationSpec.widgetName;
                    creationArgs.isMissingWidget = creationSpec.isMissingWidget;
                }
                
                var widget = new widgetClass(creationArgs, valueNode);
                
                if (presentation && presentation.displaySpec) {
                
                    var displaySpec = presentation.displaySpec;
                    
                    if (widget.getLabel && displaySpec.labelSpan) {
                        dojox.data.dom.removeChildren(displaySpec.labelSpan);
                        var text = document.createTextNode(widget.getLabel());
                        displaySpec.labelSpan.appendChild(text);
                    }
                    
                    if (displaySpec.hasDeferredLabel && displaySpec.labelSpan) {
                        if (displaySpec.labelSpan.innerHTML == "" && presentation.attributeId) { /* innerHTML OK */
                            var text = document.createTextNode(presentation.attributeId + ": ");
                            displaySpec.labelSpan.appendChild(text);
                        }
                    }
                }
                
                var i = 0;
                var id = attributeId;
                while (this._childWidgets[id]) {
                    id = attributeId + i++;
                }
                this._childWidgets[id] = widget;
            }
            if (widget && widget.setWidth && presentation.width) {
                widget.setWidth(presentation.width);
            }
            if (widget && widget.setHeight && presentation.height) {
                widget.setHeight(presentation.height);
            }
            return widget;
        },
       getPolicyWidget: function(parms){
        
            var creationSpec = this._getPresentationWidgetCreationSpec(parms.presentation);
            
            var widgetClass = creationSpec.widget;
  	        var self = this;
            if (widgetClass) {
                var creationArgs = dojo.mixin(parms,  {
                    parentWidget: this,
                    creationSpec: creationSpec,
                    parentController: this
                });
                
                if(parms.presentation.possibleValues) {
                	creationArgs._possibleValues = parms.presentation.possibleValues;
                }
                // If the widget is missing and we are not debugging, skip the widget and label
                if (creationSpec.isMissingWidget && !WorkItemUtil.isDebugMode()) 
                    return;
                
                if (creationSpec.isMissingWidget) {
                    creationArgs.widgetName = creationSpec.widgetName;
                    creationArgs.isMissingWidget = creationSpec.isMissingWidget;
                }
                
                var widget = new widgetClass(creationArgs, parms.valueNode);
                                
                var i = 0;
                var attrId = parms.presentation.attributeId || "nonattribute";
                var id = attrId;
                while (this._childWidgets[id]) {
                    id = attrId + i++;
                }
                this._childWidgets[id] = widget;
            }
            if (widget && widget.setWidth && parms.presentation.width) {
                widget.setWidth(parms.presentation.width);
            }
            if (widget && widget.setHeight && presentation.height) {
                widget.setHeight(presentation.height);
            }
            return widget;
        },

        resetChildWidgets: function() {
			for(var key in this._childWidgets) {
				if(this._childWidgets[key] && this._childWidgets[key].destroyWidget) {
					this._childWidgets[key].destroyWidget();
				} else if(this._childWidgets[key] && this._childWidgets[key].destroy) {
					this._childWidgets[key].destroy();
				}
				this._childWidgets[key] = null;
	//95375			delete this._childWidgets[key];
			}
			this._childWidgets = [];
			 
		},
        createInlineEditBox: function(text, node, handler, context, maxlength, width){
            if (!context) {
                context = this;
            }
            if (!width) {
                width = "100%";
            }
            text = EditorUtil.restoreTags(text);
            var title = document.createTextNode(text);
            node.appendChild(title);
            var inlineEdit = new dijit.InlineEditBox({
                autoSave: true,
                width: width,
                noValueIndicator: "?",
                editorParams: {
                    maxLength: maxlength
                },
                onChange: dojo.hitch(context, handler)
            }, node);
            inlineEdit.startup();
            return inlineEdit;
        },
        getViewWorkItemUri: function(id, tab){
            checkNotNull(id, "id");
            
            var idNum = parseInt(id, 10);
            checkNonNegative(idNum, "idNum");
            return "#action=com.ibm.team.workitem.viewWorkItem&id=" + id + (tab ? "&tab=" + tab : "");
        },
        /*
         * May be called by code within the com.ibm.team.workitem.web bundle.
         */
        startOperation: function(operation){
        	if (PlatformUI && PlatformUI.getWorkbench())
            	PlatformUI.getWorkbench().showMessage(operation.name);
        },
        
        /*
         * May be called by code within the com.ibm.team.workitem.web bundle.
         */
        finishOperation: function(operation){
        	if (PlatformUI && PlatformUI.getWorkbench())
            	PlatformUI.getWorkbench().hideMessageArea();
        },
        _handleDirtyingEvent: function(e){
            if (this._isDirty) 
                return;
            this.setIsDirty(true);
        },
        setIsDirty: function(isDirty){
            this._isDirty = isDirty;
        },
        handleError: function(obj, msg){
            var humanFriendlyMsg = null;
            if (!obj) {
                obj = {};
            }
            if (obj.status === 0) {
                humanFriendlyMsg = this.messages.errorServerNotResponding;
            }
            var summaryMsg = msg && dojo.isString(msg) ? msg : null;
            var detailedMsg = humanFriendlyMsg ? humanFriendlyMsg : obj.message;
            if (!detailedMsg || (detailedMsg.toLowerCase() == "not found")) {
                detailedMsg = this.messages.errorServerNotResponding;
            }
            else 
                if (detailedMsg.toLowerCase() == "unknown") {
                    detailedMsg = this.messages.errorUnknownDetails;
                }
            console.debug(dojo.string.substitute(this.messages.errorConsoleDetails, [summaryMsg ? summaryMsg : this.messages.labelUnknownError, detailedMsg]));
			
            this.showMessage(detailedMsg);
        },
        /*
         * Ignore markDirty... it is handled by the work item proxy notifications
         */
        markDirty: function() {
        	
        },
		/*
		 * Get the full version of the label, e.g. "Defect 1235: Something broke".
		 */
		getSummaryLabel: function() {
			return EditorUtil.getSummaryLabel(this.workingCopy.object);
		},
		_getMailto: function(address) {
			if(this.workingCopy.isNewWorkItem()) {
				var summaryLabel = this.getSummaryLabel();
				if(!summaryLabel ) summaryLabel = "";
				return "mailto:" + address + "?subject=" + summaryLabel;
			}
			return "mailto:" + address;
		},
        // Show the message in the header (or somewhere)
        showMessage: function(msg){
            alert(msg);
        }
    });
    
})();
